{-# OPTIONS_GHC -w #-}
module Parser where

import Char
import Processes
import Expressions
import Auxiliary
import DataSpec
import LPPE
import Debug.Trace

printExpressions [] = []
printExpressions ((var,val):rest) = (printExpression var) ++ " := " ++ (printExpression val)

data Item = DataActions [(String, ActionType)]
          | DataHiding [String] 
          | DataGlobal (Variable, Type) Expression 
          | DataRenaming [(String, String)] 
          | DataUntilFormula String
          | DataFormula Expression
          | DataEncapsulation [String]   
          | DataCommunication [(String, String, String)] 
          | DataConstant String Expression
          | DataEnumType String [String] 
          | DataRangeType String Expression Expression
          | DataFunction String [([String], String)]
          | ParserProcess Process
          | ParserInitialProcess InitialProcessDefinition
              deriving (Show, Eq)

data ActionType = None | Bool | IntRange Expression Expression deriving (Show, Eq)

data InitialProcessDefinition = InitSingleProcess InitialProcess
                              | InitParallel InitialProcessDefinition InitialProcessDefinition
                              | InitHiding [Action] InitialProcessDefinition
                              | InitEncapsulation [Action] InitialProcessDefinition
                              | InitRenaming [(Action, Action)] InitialProcessDefinition
                                   deriving (Show, Eq)

type LineNumber = Int
data ParseResult a = Ok a | Failed String
type ParserMonad a = String -> LineNumber -> ParseResult a

getLineNo :: ParserMonad LineNumber
getLineNo = \s l -> Ok l


thenParserMonad :: ParserMonad a -> (a -> ParserMonad b) -> ParserMonad b
m `thenParserMonad ` k = \s -> \l ->
   case (m s l) of 
       Ok a -> k a s l
       Failed e -> Failed e

returnParserMonad :: a -> ParserMonad a
returnParserMonad a = \s -> \l -> Ok a

failParserMonad :: String -> ParserMonad a
failParserMonad err = \s -> \l -> Failed (err ++ " on line " ++ show l)

-- parser produced by Happy Version 1.18.9

data HappyAbsSyn t4 t5 t6 t7 t8 t9 t10 t11 t12 t13 t14 t15 t16 t17 t18 t19 t20 t21 t22 t23 t24 t25 t26 t27 t28 t29 t30 t31 t32 t33 t34 t35 t36
	= HappyTerminal (Token)
	| HappyErrorToken Int
	| HappyAbsSyn4 t4
	| HappyAbsSyn5 t5
	| HappyAbsSyn6 t6
	| HappyAbsSyn7 t7
	| HappyAbsSyn8 t8
	| HappyAbsSyn9 t9
	| HappyAbsSyn10 t10
	| HappyAbsSyn11 t11
	| HappyAbsSyn12 t12
	| HappyAbsSyn13 t13
	| HappyAbsSyn14 t14
	| HappyAbsSyn15 t15
	| HappyAbsSyn16 t16
	| HappyAbsSyn17 t17
	| HappyAbsSyn18 t18
	| HappyAbsSyn19 t19
	| HappyAbsSyn20 t20
	| HappyAbsSyn21 t21
	| HappyAbsSyn22 t22
	| HappyAbsSyn23 t23
	| HappyAbsSyn24 t24
	| HappyAbsSyn25 t25
	| HappyAbsSyn26 t26
	| HappyAbsSyn27 t27
	| HappyAbsSyn28 t28
	| HappyAbsSyn29 t29
	| HappyAbsSyn30 t30
	| HappyAbsSyn31 t31
	| HappyAbsSyn32 t32
	| HappyAbsSyn33 t33
	| HappyAbsSyn34 t34
	| HappyAbsSyn35 t35
	| HappyAbsSyn36 t36

action_0 (64) = happyShift action_4
action_0 (69) = happyShift action_5
action_0 (70) = happyShift action_6
action_0 (71) = happyShift action_7
action_0 (73) = happyShift action_8
action_0 (74) = happyShift action_9
action_0 (75) = happyShift action_10
action_0 (76) = happyShift action_11
action_0 (77) = happyShift action_12
action_0 (78) = happyShift action_13
action_0 (79) = happyShift action_14
action_0 (80) = happyShift action_15
action_0 (81) = happyShift action_16
action_0 (4) = happyGoto action_17
action_0 (5) = happyGoto action_2
action_0 (6) = happyGoto action_3
action_0 _ = happyReduce_2

action_1 (64) = happyShift action_4
action_1 (69) = happyShift action_5
action_1 (70) = happyShift action_6
action_1 (71) = happyShift action_7
action_1 (73) = happyShift action_8
action_1 (74) = happyShift action_9
action_1 (75) = happyShift action_10
action_1 (76) = happyShift action_11
action_1 (77) = happyShift action_12
action_1 (78) = happyShift action_13
action_1 (79) = happyShift action_14
action_1 (80) = happyShift action_15
action_1 (81) = happyShift action_16
action_1 (5) = happyGoto action_2
action_1 (6) = happyGoto action_3
action_1 _ = happyFail

action_2 _ = happyReduce_1

action_3 (64) = happyShift action_4
action_3 (69) = happyShift action_5
action_3 (70) = happyShift action_6
action_3 (71) = happyShift action_7
action_3 (73) = happyShift action_8
action_3 (74) = happyShift action_9
action_3 (75) = happyShift action_10
action_3 (76) = happyShift action_11
action_3 (77) = happyShift action_12
action_3 (78) = happyShift action_13
action_3 (79) = happyShift action_14
action_3 (80) = happyShift action_15
action_3 (81) = happyShift action_16
action_3 (5) = happyGoto action_48
action_3 (6) = happyGoto action_3
action_3 _ = happyReduce_2

action_4 (44) = happyShift action_43
action_4 (69) = happyShift action_44
action_4 (70) = happyShift action_45
action_4 (71) = happyShift action_46
action_4 (74) = happyShift action_47
action_4 (18) = happyGoto action_42
action_4 _ = happyFail

action_5 (44) = happyShift action_40
action_5 (66) = happyShift action_41
action_5 _ = happyFail

action_6 (69) = happyShift action_32
action_6 (17) = happyGoto action_39
action_6 _ = happyFail

action_7 (44) = happyShift action_38
action_7 (15) = happyGoto action_36
action_7 (16) = happyGoto action_37
action_7 _ = happyFail

action_8 (69) = happyShift action_35
action_8 (8) = happyGoto action_33
action_8 (9) = happyGoto action_34
action_8 _ = happyFail

action_9 (69) = happyShift action_32
action_9 (17) = happyGoto action_31
action_9 _ = happyFail

action_10 (44) = happyShift action_30
action_10 (13) = happyGoto action_28
action_10 (14) = happyGoto action_29
action_10 _ = happyFail

action_11 (69) = happyShift action_27
action_11 _ = happyFail

action_12 (44) = happyShift action_23
action_12 (53) = happyShift action_24
action_12 (55) = happyShift action_25
action_12 (69) = happyShift action_26
action_12 (35) = happyGoto action_22
action_12 _ = happyFail

action_13 (69) = happyShift action_21
action_13 _ = happyFail

action_14 (42) = happyShift action_20
action_14 _ = happyFail

action_15 (69) = happyShift action_19
action_15 _ = happyFail

action_16 (69) = happyShift action_18
action_16 _ = happyFail

action_17 (82) = happyAccept
action_17 _ = happyFail

action_18 (66) = happyShift action_101
action_18 _ = happyFail

action_19 (66) = happyShift action_100
action_19 _ = happyFail

action_20 (39) = happyShift action_94
action_20 (44) = happyShift action_95
action_20 (45) = happyShift action_96
action_20 (48) = happyShift action_97
action_20 (53) = happyShift action_98
action_20 (69) = happyShift action_99
action_20 (7) = happyGoto action_93
action_20 _ = happyReduce_19

action_21 (58) = happyShift action_92
action_21 _ = happyFail

action_22 (45) = happyShift action_79
action_22 (49) = happyShift action_80
action_22 (50) = happyShift action_81
action_22 (52) = happyShift action_82
action_22 (54) = happyShift action_83
action_22 (55) = happyShift action_84
action_22 (56) = happyShift action_85
action_22 (57) = happyShift action_86
action_22 (60) = happyShift action_87
action_22 (61) = happyShift action_88
action_22 (63) = happyShift action_89
action_22 (65) = happyShift action_90
action_22 (66) = happyShift action_91
action_22 _ = happyReduce_11

action_23 (44) = happyShift action_23
action_23 (53) = happyShift action_24
action_23 (55) = happyShift action_25
action_23 (69) = happyShift action_26
action_23 (35) = happyGoto action_78
action_23 _ = happyFail

action_24 (44) = happyShift action_23
action_24 (53) = happyShift action_24
action_24 (55) = happyShift action_25
action_24 (69) = happyShift action_26
action_24 (35) = happyGoto action_77
action_24 _ = happyFail

action_25 (44) = happyShift action_23
action_25 (53) = happyShift action_24
action_25 (55) = happyShift action_25
action_25 (69) = happyShift action_26
action_25 (35) = happyGoto action_76
action_25 _ = happyFail

action_26 (44) = happyShift action_75
action_26 _ = happyReduce_105

action_27 (66) = happyShift action_74
action_27 _ = happyFail

action_28 _ = happyReduce_9

action_29 (39) = happyShift action_73
action_29 _ = happyReduce_36

action_30 (69) = happyShift action_72
action_30 _ = happyFail

action_31 _ = happyReduce_8

action_32 (39) = happyShift action_71
action_32 _ = happyReduce_42

action_33 _ = happyReduce_6

action_34 (39) = happyShift action_70
action_34 _ = happyReduce_26

action_35 (58) = happyShift action_69
action_35 _ = happyReduce_28

action_36 _ = happyReduce_5

action_37 (39) = happyShift action_68
action_37 _ = happyReduce_39

action_38 (69) = happyShift action_67
action_38 _ = happyFail

action_39 _ = happyReduce_4

action_40 (69) = happyShift action_66
action_40 (26) = happyGoto action_64
action_40 (27) = happyGoto action_65
action_40 _ = happyFail

action_41 (37) = happyShift action_60
action_41 (44) = happyShift action_61
action_41 (53) = happyShift action_24
action_41 (55) = happyShift action_25
action_41 (60) = happyShift action_62
action_41 (69) = happyShift action_63
action_41 (21) = happyGoto action_57
action_41 (22) = happyGoto action_58
action_41 (35) = happyGoto action_59
action_41 _ = happyFail

action_42 (41) = happyShift action_56
action_42 _ = happyReduce_16

action_43 (44) = happyShift action_43
action_43 (69) = happyShift action_44
action_43 (70) = happyShift action_45
action_43 (71) = happyShift action_46
action_43 (74) = happyShift action_47
action_43 (18) = happyGoto action_55
action_43 _ = happyFail

action_44 (42) = happyShift action_53
action_44 (44) = happyShift action_54
action_44 (20) = happyGoto action_52
action_44 _ = happyReduce_54

action_45 (44) = happyShift action_51
action_45 _ = happyFail

action_46 (44) = happyShift action_50
action_46 _ = happyFail

action_47 (44) = happyShift action_49
action_47 _ = happyFail

action_48 _ = happyReduce_3

action_49 (69) = happyShift action_32
action_49 (17) = happyGoto action_161
action_49 _ = happyFail

action_50 (44) = happyShift action_38
action_50 (15) = happyGoto action_160
action_50 (16) = happyGoto action_37
action_50 _ = happyFail

action_51 (69) = happyShift action_32
action_51 (17) = happyGoto action_159
action_51 _ = happyFail

action_52 _ = happyReduce_44

action_53 (43) = happyShift action_158
action_53 (44) = happyShift action_23
action_53 (53) = happyShift action_24
action_53 (55) = happyShift action_25
action_53 (69) = happyShift action_26
action_53 (34) = happyGoto action_157
action_53 (35) = happyGoto action_130
action_53 _ = happyFail

action_54 (69) = happyShift action_156
action_54 (30) = happyGoto action_154
action_54 (31) = happyGoto action_155
action_54 _ = happyFail

action_55 (41) = happyShift action_56
action_55 (48) = happyShift action_153
action_55 _ = happyFail

action_56 (44) = happyShift action_43
action_56 (69) = happyShift action_44
action_56 (70) = happyShift action_45
action_56 (71) = happyShift action_46
action_56 (74) = happyShift action_47
action_56 (18) = happyGoto action_152
action_56 _ = happyFail

action_57 (51) = happyShift action_151
action_57 _ = happyReduce_18

action_58 _ = happyReduce_64

action_59 (45) = happyShift action_79
action_59 (49) = happyShift action_80
action_59 (50) = happyShift action_81
action_59 (52) = happyShift action_82
action_59 (54) = happyShift action_83
action_59 (55) = happyShift action_84
action_59 (56) = happyShift action_85
action_59 (57) = happyShift action_86
action_59 (60) = happyShift action_87
action_59 (61) = happyShift action_88
action_59 (63) = happyShift action_89
action_59 (65) = happyShift action_90
action_59 (66) = happyShift action_91
action_59 (67) = happyShift action_150
action_59 _ = happyFail

action_60 (44) = happyShift action_149
action_60 _ = happyFail

action_61 (37) = happyShift action_60
action_61 (44) = happyShift action_61
action_61 (53) = happyShift action_24
action_61 (55) = happyShift action_25
action_61 (60) = happyShift action_62
action_61 (69) = happyShift action_63
action_61 (21) = happyGoto action_147
action_61 (22) = happyGoto action_58
action_61 (35) = happyGoto action_148
action_61 _ = happyFail

action_62 (44) = happyShift action_23
action_62 (53) = happyShift action_24
action_62 (55) = happyShift action_25
action_62 (69) = happyShift action_26
action_62 (35) = happyGoto action_146
action_62 _ = happyFail

action_63 (42) = happyShift action_144
action_63 (44) = happyShift action_145
action_63 (45) = happyReduce_105
action_63 (48) = happyReduce_105
action_63 (49) = happyReduce_105
action_63 (50) = happyReduce_105
action_63 (52) = happyReduce_105
action_63 (54) = happyReduce_105
action_63 (55) = happyReduce_105
action_63 (56) = happyReduce_105
action_63 (57) = happyReduce_105
action_63 (60) = happyReduce_105
action_63 (61) = happyReduce_105
action_63 (63) = happyReduce_105
action_63 (65) = happyReduce_105
action_63 (66) = happyReduce_105
action_63 (67) = happyReduce_105
action_63 (68) = happyReduce_105
action_63 (19) = happyGoto action_143
action_63 _ = happyReduce_51

action_64 (39) = happyShift action_141
action_64 (48) = happyShift action_142
action_64 _ = happyFail

action_65 _ = happyReduce_75

action_66 (58) = happyShift action_140
action_66 _ = happyFail

action_67 (39) = happyShift action_139
action_67 _ = happyFail

action_68 (44) = happyShift action_38
action_68 (15) = happyGoto action_138
action_68 (16) = happyGoto action_37
action_68 _ = happyFail

action_69 (46) = happyShift action_136
action_69 (72) = happyShift action_137
action_69 _ = happyFail

action_70 (69) = happyShift action_35
action_70 (8) = happyGoto action_135
action_70 (9) = happyGoto action_34
action_70 _ = happyFail

action_71 (69) = happyShift action_32
action_71 (17) = happyGoto action_134
action_71 _ = happyFail

action_72 (39) = happyShift action_133
action_72 _ = happyFail

action_73 (44) = happyShift action_30
action_73 (13) = happyGoto action_132
action_73 (14) = happyGoto action_29
action_73 _ = happyFail

action_74 (44) = happyShift action_23
action_74 (53) = happyShift action_24
action_74 (55) = happyShift action_25
action_74 (69) = happyShift action_26
action_74 (35) = happyGoto action_131
action_74 _ = happyFail

action_75 (44) = happyShift action_23
action_75 (53) = happyShift action_24
action_75 (55) = happyShift action_25
action_75 (69) = happyShift action_26
action_75 (34) = happyGoto action_129
action_75 (35) = happyGoto action_130
action_75 _ = happyFail

action_76 _ = happyReduce_101

action_77 (49) = happyShift action_80
action_77 (50) = happyShift action_81
action_77 (54) = happyShift action_83
action_77 (55) = happyShift action_84
action_77 (56) = happyShift action_85
action_77 (57) = happyShift action_86
action_77 (60) = happyShift action_87
action_77 (61) = happyShift action_88
action_77 (63) = happyShift action_89
action_77 (65) = happyShift action_90
action_77 (66) = happyShift action_91
action_77 _ = happyReduce_97

action_78 (45) = happyShift action_79
action_78 (48) = happyShift action_128
action_78 (49) = happyShift action_80
action_78 (50) = happyShift action_81
action_78 (52) = happyShift action_82
action_78 (54) = happyShift action_83
action_78 (55) = happyShift action_84
action_78 (56) = happyShift action_85
action_78 (57) = happyShift action_86
action_78 (60) = happyShift action_87
action_78 (61) = happyShift action_88
action_78 (63) = happyShift action_89
action_78 (65) = happyShift action_90
action_78 (66) = happyShift action_91
action_78 _ = happyFail

action_79 (44) = happyShift action_23
action_79 (53) = happyShift action_24
action_79 (55) = happyShift action_25
action_79 (69) = happyShift action_26
action_79 (35) = happyGoto action_127
action_79 _ = happyFail

action_80 (44) = happyShift action_23
action_80 (53) = happyShift action_24
action_80 (55) = happyShift action_25
action_80 (69) = happyShift action_26
action_80 (35) = happyGoto action_126
action_80 _ = happyFail

action_81 (44) = happyShift action_23
action_81 (53) = happyShift action_24
action_81 (55) = happyShift action_25
action_81 (69) = happyShift action_26
action_81 (35) = happyGoto action_125
action_81 _ = happyFail

action_82 (44) = happyShift action_23
action_82 (53) = happyShift action_24
action_82 (55) = happyShift action_25
action_82 (69) = happyShift action_26
action_82 (35) = happyGoto action_124
action_82 _ = happyFail

action_83 (44) = happyShift action_23
action_83 (53) = happyShift action_24
action_83 (55) = happyShift action_25
action_83 (69) = happyShift action_26
action_83 (35) = happyGoto action_123
action_83 _ = happyFail

action_84 (44) = happyShift action_23
action_84 (53) = happyShift action_24
action_84 (55) = happyShift action_25
action_84 (69) = happyShift action_26
action_84 (35) = happyGoto action_122
action_84 _ = happyFail

action_85 (44) = happyShift action_23
action_85 (53) = happyShift action_24
action_85 (55) = happyShift action_25
action_85 (69) = happyShift action_26
action_85 (35) = happyGoto action_121
action_85 _ = happyFail

action_86 (44) = happyShift action_23
action_86 (53) = happyShift action_24
action_86 (55) = happyShift action_25
action_86 (69) = happyShift action_26
action_86 (35) = happyGoto action_120
action_86 _ = happyFail

action_87 (44) = happyShift action_23
action_87 (53) = happyShift action_24
action_87 (55) = happyShift action_25
action_87 (69) = happyShift action_26
action_87 (35) = happyGoto action_119
action_87 _ = happyFail

action_88 (44) = happyShift action_23
action_88 (53) = happyShift action_24
action_88 (55) = happyShift action_25
action_88 (69) = happyShift action_26
action_88 (35) = happyGoto action_118
action_88 _ = happyFail

action_89 (44) = happyShift action_23
action_89 (53) = happyShift action_24
action_89 (55) = happyShift action_25
action_89 (69) = happyShift action_26
action_89 (35) = happyGoto action_117
action_89 _ = happyFail

action_90 (44) = happyShift action_23
action_90 (53) = happyShift action_24
action_90 (55) = happyShift action_25
action_90 (69) = happyShift action_26
action_90 (35) = happyGoto action_116
action_90 _ = happyFail

action_91 (44) = happyShift action_23
action_91 (53) = happyShift action_24
action_91 (55) = happyShift action_25
action_91 (69) = happyShift action_26
action_91 (35) = happyGoto action_115
action_91 _ = happyFail

action_92 (46) = happyShift action_113
action_92 (69) = happyShift action_114
action_92 (23) = happyGoto action_112
action_92 _ = happyFail

action_93 (43) = happyShift action_111
action_93 _ = happyFail

action_94 (39) = happyShift action_94
action_94 (44) = happyShift action_95
action_94 (45) = happyShift action_96
action_94 (48) = happyShift action_97
action_94 (53) = happyShift action_98
action_94 (69) = happyShift action_99
action_94 (7) = happyGoto action_110
action_94 _ = happyReduce_19

action_95 (39) = happyShift action_94
action_95 (44) = happyShift action_95
action_95 (45) = happyShift action_96
action_95 (48) = happyShift action_97
action_95 (53) = happyShift action_98
action_95 (69) = happyShift action_99
action_95 (7) = happyGoto action_109
action_95 _ = happyReduce_19

action_96 (39) = happyShift action_94
action_96 (44) = happyShift action_95
action_96 (45) = happyShift action_96
action_96 (48) = happyShift action_97
action_96 (53) = happyShift action_98
action_96 (69) = happyShift action_99
action_96 (7) = happyGoto action_108
action_96 _ = happyReduce_19

action_97 (39) = happyShift action_94
action_97 (44) = happyShift action_95
action_97 (45) = happyShift action_96
action_97 (48) = happyShift action_97
action_97 (53) = happyShift action_98
action_97 (69) = happyShift action_99
action_97 (7) = happyGoto action_107
action_97 _ = happyReduce_19

action_98 (39) = happyShift action_94
action_98 (44) = happyShift action_95
action_98 (45) = happyShift action_96
action_98 (48) = happyShift action_97
action_98 (53) = happyShift action_98
action_98 (69) = happyShift action_99
action_98 (7) = happyGoto action_106
action_98 _ = happyReduce_19

action_99 (39) = happyShift action_94
action_99 (44) = happyShift action_95
action_99 (45) = happyShift action_96
action_99 (48) = happyShift action_97
action_99 (53) = happyShift action_98
action_99 (69) = happyShift action_99
action_99 (7) = happyGoto action_105
action_99 _ = happyReduce_19

action_100 (44) = happyShift action_103
action_100 (46) = happyShift action_104
action_100 _ = happyFail

action_101 (44) = happyShift action_102
action_101 _ = happyFail

action_102 (69) = happyShift action_198
action_102 (10) = happyGoto action_195
action_102 (11) = happyGoto action_196
action_102 (12) = happyGoto action_197
action_102 _ = happyFail

action_103 (69) = happyShift action_32
action_103 (17) = happyGoto action_194
action_103 _ = happyFail

action_104 (44) = happyShift action_23
action_104 (53) = happyShift action_24
action_104 (55) = happyShift action_25
action_104 (69) = happyShift action_26
action_104 (35) = happyGoto action_193
action_104 _ = happyFail

action_105 _ = happyReduce_20

action_106 _ = happyReduce_22

action_107 _ = happyReduce_24

action_108 _ = happyReduce_21

action_109 _ = happyReduce_23

action_110 _ = happyReduce_25

action_111 _ = happyReduce_10

action_112 (66) = happyShift action_192
action_112 _ = happyFail

action_113 (44) = happyShift action_23
action_113 (53) = happyShift action_24
action_113 (55) = happyShift action_25
action_113 (69) = happyShift action_26
action_113 (35) = happyGoto action_191
action_113 _ = happyFail

action_114 _ = happyReduce_70

action_115 (49) = happyShift action_80
action_115 (50) = happyShift action_81
action_115 (54) = happyFail
action_115 (55) = happyShift action_84
action_115 (56) = happyShift action_85
action_115 (57) = happyShift action_86
action_115 (60) = happyShift action_87
action_115 (61) = happyShift action_88
action_115 (63) = happyShift action_89
action_115 (65) = happyShift action_90
action_115 (66) = happyFail
action_115 _ = happyReduce_95

action_116 (49) = happyShift action_80
action_116 (50) = happyShift action_81
action_116 (55) = happyShift action_84
action_116 (56) = happyShift action_85
action_116 (57) = happyShift action_86
action_116 (60) = happyFail
action_116 (61) = happyFail
action_116 (63) = happyFail
action_116 (65) = happyFail
action_116 _ = happyReduce_94

action_117 (49) = happyShift action_80
action_117 (50) = happyShift action_81
action_117 (55) = happyShift action_84
action_117 (56) = happyShift action_85
action_117 (57) = happyShift action_86
action_117 (60) = happyFail
action_117 (61) = happyFail
action_117 (63) = happyFail
action_117 (65) = happyFail
action_117 _ = happyReduce_90

action_118 (49) = happyShift action_80
action_118 (50) = happyShift action_81
action_118 (55) = happyShift action_84
action_118 (56) = happyShift action_85
action_118 (57) = happyShift action_86
action_118 (60) = happyFail
action_118 (61) = happyFail
action_118 (63) = happyFail
action_118 (65) = happyFail
action_118 _ = happyReduce_91

action_119 (49) = happyShift action_80
action_119 (50) = happyShift action_81
action_119 (55) = happyShift action_84
action_119 (56) = happyShift action_85
action_119 (57) = happyShift action_86
action_119 (60) = happyFail
action_119 (61) = happyFail
action_119 (63) = happyFail
action_119 (65) = happyFail
action_119 _ = happyReduce_89

action_120 (50) = happyShift action_81
action_120 _ = happyReduce_102

action_121 (50) = happyShift action_81
action_121 _ = happyReduce_103

action_122 (50) = happyShift action_81
action_122 (56) = happyShift action_85
action_122 (57) = happyShift action_86
action_122 _ = happyReduce_100

action_123 (49) = happyShift action_80
action_123 (50) = happyShift action_81
action_123 (54) = happyFail
action_123 (55) = happyShift action_84
action_123 (56) = happyShift action_85
action_123 (57) = happyShift action_86
action_123 (60) = happyShift action_87
action_123 (61) = happyShift action_88
action_123 (63) = happyShift action_89
action_123 (65) = happyShift action_90
action_123 (66) = happyFail
action_123 _ = happyReduce_96

action_124 (49) = happyShift action_80
action_124 (50) = happyShift action_81
action_124 (54) = happyShift action_83
action_124 (55) = happyShift action_84
action_124 (56) = happyShift action_85
action_124 (57) = happyShift action_86
action_124 (60) = happyShift action_87
action_124 (61) = happyShift action_88
action_124 (63) = happyShift action_89
action_124 (65) = happyShift action_90
action_124 (66) = happyShift action_91
action_124 _ = happyReduce_92

action_125 _ = happyReduce_99

action_126 (50) = happyShift action_81
action_126 (56) = happyShift action_85
action_126 (57) = happyShift action_86
action_126 _ = happyReduce_98

action_127 (49) = happyShift action_80
action_127 (50) = happyShift action_81
action_127 (52) = happyShift action_82
action_127 (54) = happyShift action_83
action_127 (55) = happyShift action_84
action_127 (56) = happyShift action_85
action_127 (57) = happyShift action_86
action_127 (60) = happyShift action_87
action_127 (61) = happyShift action_88
action_127 (63) = happyShift action_89
action_127 (65) = happyShift action_90
action_127 (66) = happyShift action_91
action_127 _ = happyReduce_93

action_128 _ = happyReduce_106

action_129 (39) = happyShift action_165
action_129 (48) = happyShift action_190
action_129 _ = happyFail

action_130 (45) = happyShift action_79
action_130 (49) = happyShift action_80
action_130 (50) = happyShift action_81
action_130 (52) = happyShift action_82
action_130 (54) = happyShift action_83
action_130 (55) = happyShift action_84
action_130 (56) = happyShift action_85
action_130 (57) = happyShift action_86
action_130 (60) = happyShift action_87
action_130 (61) = happyShift action_88
action_130 (63) = happyShift action_89
action_130 (65) = happyShift action_90
action_130 (66) = happyShift action_91
action_130 _ = happyReduce_87

action_131 (45) = happyShift action_79
action_131 (49) = happyShift action_80
action_131 (50) = happyShift action_81
action_131 (52) = happyShift action_82
action_131 (54) = happyShift action_83
action_131 (55) = happyShift action_84
action_131 (56) = happyShift action_85
action_131 (57) = happyShift action_86
action_131 (60) = happyShift action_87
action_131 (61) = happyShift action_88
action_131 (63) = happyShift action_89
action_131 (65) = happyShift action_90
action_131 (66) = happyShift action_91
action_131 _ = happyReduce_12

action_132 _ = happyReduce_37

action_133 (69) = happyShift action_189
action_133 _ = happyFail

action_134 _ = happyReduce_43

action_135 _ = happyReduce_27

action_136 (44) = happyShift action_23
action_136 (53) = happyShift action_24
action_136 (55) = happyShift action_25
action_136 (69) = happyShift action_26
action_136 (35) = happyGoto action_188
action_136 _ = happyFail

action_137 _ = happyReduce_29

action_138 _ = happyReduce_40

action_139 (69) = happyShift action_187
action_139 _ = happyFail

action_140 (46) = happyShift action_113
action_140 (69) = happyShift action_114
action_140 (23) = happyGoto action_186
action_140 _ = happyFail

action_141 (69) = happyShift action_66
action_141 (27) = happyGoto action_185
action_141 _ = happyFail

action_142 (66) = happyShift action_184
action_142 _ = happyFail

action_143 (40) = happyShift action_182
action_143 (46) = happyShift action_183
action_143 _ = happyFail

action_144 (43) = happyShift action_180
action_144 (44) = happyShift action_23
action_144 (53) = happyShift action_24
action_144 (55) = happyShift action_25
action_144 (69) = happyShift action_181
action_144 (28) = happyGoto action_177
action_144 (29) = happyGoto action_178
action_144 (34) = happyGoto action_179
action_144 (35) = happyGoto action_130
action_144 _ = happyFail

action_145 (44) = happyShift action_23
action_145 (48) = happyShift action_176
action_145 (53) = happyShift action_24
action_145 (55) = happyShift action_25
action_145 (69) = happyShift action_26
action_145 (34) = happyGoto action_175
action_145 (35) = happyGoto action_130
action_145 _ = happyFail

action_146 (45) = happyShift action_79
action_146 (49) = happyShift action_80
action_146 (50) = happyShift action_81
action_146 (52) = happyShift action_82
action_146 (54) = happyShift action_83
action_146 (55) = happyShift action_84
action_146 (56) = happyShift action_85
action_146 (57) = happyShift action_86
action_146 (60) = happyShift action_87
action_146 (61) = happyShift action_174
action_146 (63) = happyShift action_89
action_146 (65) = happyShift action_90
action_146 (66) = happyShift action_91
action_146 _ = happyFail

action_147 (48) = happyShift action_173
action_147 (51) = happyShift action_151
action_147 _ = happyFail

action_148 (45) = happyShift action_79
action_148 (48) = happyShift action_128
action_148 (49) = happyShift action_80
action_148 (50) = happyShift action_81
action_148 (52) = happyShift action_82
action_148 (54) = happyShift action_83
action_148 (55) = happyShift action_84
action_148 (56) = happyShift action_85
action_148 (57) = happyShift action_86
action_148 (60) = happyShift action_87
action_148 (61) = happyShift action_88
action_148 (63) = happyShift action_89
action_148 (65) = happyShift action_90
action_148 (66) = happyShift action_91
action_148 (67) = happyShift action_150
action_148 _ = happyFail

action_149 (69) = happyShift action_172
action_149 _ = happyFail

action_150 (37) = happyShift action_60
action_150 (44) = happyShift action_61
action_150 (53) = happyShift action_24
action_150 (55) = happyShift action_25
action_150 (60) = happyShift action_62
action_150 (69) = happyShift action_63
action_150 (21) = happyGoto action_171
action_150 (22) = happyGoto action_58
action_150 (35) = happyGoto action_59
action_150 _ = happyFail

action_151 (37) = happyShift action_60
action_151 (44) = happyShift action_61
action_151 (53) = happyShift action_24
action_151 (55) = happyShift action_25
action_151 (60) = happyShift action_62
action_151 (69) = happyShift action_63
action_151 (21) = happyGoto action_170
action_151 (22) = happyGoto action_58
action_151 (35) = happyGoto action_59
action_151 _ = happyFail

action_152 _ = happyReduce_46

action_153 _ = happyReduce_50

action_154 (39) = happyShift action_168
action_154 (48) = happyShift action_169
action_154 _ = happyFail

action_155 _ = happyReduce_81

action_156 (66) = happyShift action_167
action_156 _ = happyFail

action_157 (39) = happyShift action_165
action_157 (43) = happyShift action_166
action_157 _ = happyFail

action_158 _ = happyReduce_55

action_159 (58) = happyShift action_164
action_159 _ = happyFail

action_160 (58) = happyShift action_163
action_160 _ = happyFail

action_161 (58) = happyShift action_162
action_161 _ = happyFail

action_162 (44) = happyShift action_43
action_162 (69) = happyShift action_44
action_162 (70) = happyShift action_45
action_162 (71) = happyShift action_46
action_162 (74) = happyShift action_47
action_162 (18) = happyGoto action_229
action_162 _ = happyFail

action_163 (44) = happyShift action_43
action_163 (69) = happyShift action_44
action_163 (70) = happyShift action_45
action_163 (71) = happyShift action_46
action_163 (74) = happyShift action_47
action_163 (18) = happyGoto action_228
action_163 _ = happyFail

action_164 (44) = happyShift action_43
action_164 (69) = happyShift action_44
action_164 (70) = happyShift action_45
action_164 (71) = happyShift action_46
action_164 (74) = happyShift action_47
action_164 (18) = happyGoto action_227
action_164 _ = happyFail

action_165 (44) = happyShift action_23
action_165 (53) = happyShift action_24
action_165 (55) = happyShift action_25
action_165 (69) = happyShift action_26
action_165 (35) = happyGoto action_226
action_165 _ = happyFail

action_166 _ = happyReduce_56

action_167 (69) = happyShift action_225
action_167 _ = happyFail

action_168 (69) = happyShift action_156
action_168 (31) = happyGoto action_224
action_168 _ = happyFail

action_169 (42) = happyShift action_53
action_169 (20) = happyGoto action_223
action_169 _ = happyReduce_54

action_170 _ = happyReduce_57

action_171 _ = happyReduce_63

action_172 (58) = happyShift action_222
action_172 _ = happyFail

action_173 _ = happyReduce_65

action_174 (40) = happyShift action_221
action_174 (44) = happyShift action_23
action_174 (53) = happyShift action_24
action_174 (55) = happyShift action_25
action_174 (69) = happyShift action_26
action_174 (35) = happyGoto action_118
action_174 _ = happyFail

action_175 (39) = happyShift action_165
action_175 (48) = happyShift action_220
action_175 _ = happyFail

action_176 _ = happyReduce_52

action_177 (39) = happyShift action_218
action_177 (43) = happyShift action_219
action_177 _ = happyFail

action_178 _ = happyReduce_78

action_179 (39) = happyShift action_165
action_179 (43) = happyShift action_217
action_179 _ = happyFail

action_180 _ = happyReduce_69

action_181 (44) = happyShift action_75
action_181 (59) = happyShift action_216
action_181 _ = happyReduce_105

action_182 (37) = happyShift action_60
action_182 (38) = happyShift action_215
action_182 (44) = happyShift action_61
action_182 (53) = happyShift action_24
action_182 (55) = happyShift action_25
action_182 (60) = happyShift action_62
action_182 (69) = happyShift action_63
action_182 (21) = happyGoto action_214
action_182 (22) = happyGoto action_58
action_182 (35) = happyGoto action_59
action_182 _ = happyFail

action_183 (44) = happyShift action_23
action_183 (53) = happyShift action_24
action_183 (55) = happyShift action_25
action_183 (69) = happyShift action_26
action_183 (32) = happyGoto action_211
action_183 (33) = happyGoto action_212
action_183 (35) = happyGoto action_213
action_183 _ = happyFail

action_184 (37) = happyShift action_60
action_184 (44) = happyShift action_61
action_184 (53) = happyShift action_24
action_184 (55) = happyShift action_25
action_184 (60) = happyShift action_62
action_184 (69) = happyShift action_63
action_184 (21) = happyGoto action_210
action_184 (22) = happyGoto action_58
action_184 (35) = happyGoto action_59
action_184 _ = happyFail

action_185 _ = happyReduce_76

action_186 _ = happyReduce_77

action_187 (48) = happyShift action_209
action_187 _ = happyFail

action_188 (45) = happyShift action_79
action_188 (49) = happyShift action_80
action_188 (50) = happyShift action_81
action_188 (52) = happyShift action_82
action_188 (54) = happyShift action_83
action_188 (55) = happyShift action_84
action_188 (56) = happyShift action_85
action_188 (57) = happyShift action_86
action_188 (60) = happyShift action_87
action_188 (61) = happyShift action_88
action_188 (62) = happyShift action_208
action_188 (63) = happyShift action_89
action_188 (65) = happyShift action_90
action_188 (66) = happyShift action_91
action_188 _ = happyFail

action_189 (39) = happyShift action_207
action_189 _ = happyFail

action_190 _ = happyReduce_104

action_191 (45) = happyShift action_79
action_191 (49) = happyShift action_80
action_191 (50) = happyShift action_81
action_191 (52) = happyShift action_82
action_191 (54) = happyShift action_83
action_191 (55) = happyShift action_84
action_191 (56) = happyShift action_85
action_191 (57) = happyShift action_86
action_191 (60) = happyShift action_87
action_191 (61) = happyShift action_88
action_191 (62) = happyShift action_206
action_191 (63) = happyShift action_89
action_191 (65) = happyShift action_90
action_191 (66) = happyShift action_91
action_191 _ = happyFail

action_192 (44) = happyShift action_23
action_192 (53) = happyShift action_24
action_192 (55) = happyShift action_25
action_192 (69) = happyShift action_26
action_192 (35) = happyGoto action_205
action_192 _ = happyFail

action_193 (45) = happyShift action_79
action_193 (49) = happyShift action_80
action_193 (50) = happyShift action_81
action_193 (52) = happyShift action_82
action_193 (54) = happyShift action_83
action_193 (55) = happyShift action_84
action_193 (56) = happyShift action_85
action_193 (57) = happyShift action_86
action_193 (60) = happyShift action_87
action_193 (61) = happyShift action_88
action_193 (62) = happyShift action_204
action_193 (63) = happyShift action_89
action_193 (65) = happyShift action_90
action_193 (66) = happyShift action_91
action_193 _ = happyFail

action_194 (48) = happyShift action_203
action_194 _ = happyFail

action_195 (48) = happyShift action_202
action_195 _ = happyFail

action_196 (39) = happyShift action_201
action_196 _ = happyReduce_31

action_197 (68) = happyShift action_200
action_197 _ = happyFail

action_198 (57) = happyShift action_199
action_198 _ = happyReduce_34

action_199 (69) = happyShift action_198
action_199 (12) = happyGoto action_248
action_199 _ = happyFail

action_200 (69) = happyShift action_247
action_200 _ = happyFail

action_201 (69) = happyShift action_198
action_201 (10) = happyGoto action_246
action_201 (11) = happyGoto action_196
action_201 (12) = happyGoto action_197
action_201 _ = happyFail

action_202 _ = happyReduce_15

action_203 _ = happyReduce_13

action_204 (44) = happyShift action_23
action_204 (53) = happyShift action_24
action_204 (55) = happyShift action_25
action_204 (69) = happyShift action_26
action_204 (35) = happyGoto action_245
action_204 _ = happyFail

action_205 (45) = happyShift action_79
action_205 (49) = happyShift action_80
action_205 (50) = happyShift action_81
action_205 (52) = happyShift action_82
action_205 (54) = happyShift action_83
action_205 (55) = happyShift action_84
action_205 (56) = happyShift action_85
action_205 (57) = happyShift action_86
action_205 (60) = happyShift action_87
action_205 (61) = happyShift action_88
action_205 (63) = happyShift action_89
action_205 (65) = happyShift action_90
action_205 (66) = happyShift action_91
action_205 _ = happyReduce_7

action_206 (44) = happyShift action_23
action_206 (53) = happyShift action_24
action_206 (55) = happyShift action_25
action_206 (69) = happyShift action_26
action_206 (35) = happyGoto action_244
action_206 _ = happyFail

action_207 (69) = happyShift action_243
action_207 _ = happyFail

action_208 (44) = happyShift action_23
action_208 (53) = happyShift action_24
action_208 (55) = happyShift action_25
action_208 (69) = happyShift action_26
action_208 (35) = happyGoto action_242
action_208 _ = happyFail

action_209 _ = happyReduce_41

action_210 (51) = happyShift action_151
action_210 _ = happyReduce_17

action_211 (39) = happyShift action_240
action_211 (47) = happyShift action_241
action_211 _ = happyFail

action_212 _ = happyReduce_84

action_213 (45) = happyShift action_79
action_213 (49) = happyShift action_80
action_213 (50) = happyShift action_81
action_213 (52) = happyShift action_82
action_213 (54) = happyShift action_83
action_213 (55) = happyShift action_84
action_213 (56) = happyShift action_85
action_213 (57) = happyShift action_86
action_213 (59) = happyShift action_239
action_213 (60) = happyShift action_87
action_213 (61) = happyShift action_88
action_213 (63) = happyShift action_89
action_213 (65) = happyShift action_90
action_213 (66) = happyShift action_91
action_213 _ = happyFail

action_214 _ = happyReduce_58

action_215 (44) = happyShift action_238
action_215 _ = happyFail

action_216 (44) = happyShift action_23
action_216 (53) = happyShift action_24
action_216 (55) = happyShift action_25
action_216 (69) = happyShift action_26
action_216 (35) = happyGoto action_237
action_216 _ = happyFail

action_217 _ = happyReduce_67

action_218 (69) = happyShift action_236
action_218 (29) = happyGoto action_235
action_218 _ = happyFail

action_219 _ = happyReduce_68

action_220 (45) = happyReduce_104
action_220 (48) = happyReduce_104
action_220 (49) = happyReduce_104
action_220 (50) = happyReduce_104
action_220 (52) = happyReduce_104
action_220 (54) = happyReduce_104
action_220 (55) = happyReduce_104
action_220 (56) = happyReduce_104
action_220 (57) = happyReduce_104
action_220 (60) = happyReduce_104
action_220 (61) = happyReduce_104
action_220 (63) = happyReduce_104
action_220 (65) = happyReduce_104
action_220 (66) = happyReduce_104
action_220 (67) = happyReduce_104
action_220 (68) = happyReduce_104
action_220 _ = happyReduce_53

action_221 (37) = happyShift action_60
action_221 (44) = happyShift action_61
action_221 (53) = happyShift action_24
action_221 (55) = happyShift action_25
action_221 (60) = happyShift action_62
action_221 (69) = happyShift action_63
action_221 (21) = happyGoto action_234
action_221 (22) = happyGoto action_58
action_221 (35) = happyGoto action_59
action_221 _ = happyFail

action_222 (46) = happyShift action_113
action_222 (69) = happyShift action_114
action_222 (23) = happyGoto action_233
action_222 _ = happyFail

action_223 _ = happyReduce_45

action_224 _ = happyReduce_82

action_225 _ = happyReduce_83

action_226 (45) = happyShift action_79
action_226 (49) = happyShift action_80
action_226 (50) = happyShift action_81
action_226 (52) = happyShift action_82
action_226 (54) = happyShift action_83
action_226 (55) = happyShift action_84
action_226 (56) = happyShift action_85
action_226 (57) = happyShift action_86
action_226 (60) = happyShift action_87
action_226 (61) = happyShift action_88
action_226 (63) = happyShift action_89
action_226 (65) = happyShift action_90
action_226 (66) = happyShift action_91
action_226 _ = happyReduce_88

action_227 (41) = happyShift action_56
action_227 (48) = happyShift action_232
action_227 _ = happyFail

action_228 (41) = happyShift action_56
action_228 (48) = happyShift action_231
action_228 _ = happyFail

action_229 (41) = happyShift action_56
action_229 (48) = happyShift action_230
action_229 _ = happyFail

action_230 _ = happyReduce_48

action_231 _ = happyReduce_49

action_232 _ = happyReduce_47

action_233 (39) = happyShift action_261
action_233 _ = happyFail

action_234 _ = happyReduce_66

action_235 _ = happyReduce_79

action_236 (59) = happyShift action_216
action_236 _ = happyFail

action_237 (45) = happyShift action_79
action_237 (49) = happyShift action_80
action_237 (50) = happyShift action_81
action_237 (52) = happyShift action_82
action_237 (54) = happyShift action_83
action_237 (55) = happyShift action_84
action_237 (56) = happyShift action_85
action_237 (57) = happyShift action_86
action_237 (60) = happyShift action_87
action_237 (61) = happyShift action_88
action_237 (63) = happyShift action_89
action_237 (65) = happyShift action_90
action_237 (66) = happyShift action_91
action_237 _ = happyReduce_80

action_238 (44) = happyShift action_23
action_238 (53) = happyShift action_24
action_238 (55) = happyShift action_25
action_238 (69) = happyShift action_260
action_238 (24) = happyGoto action_256
action_238 (25) = happyGoto action_257
action_238 (35) = happyGoto action_258
action_238 (36) = happyGoto action_259
action_238 _ = happyFail

action_239 (44) = happyShift action_23
action_239 (53) = happyShift action_24
action_239 (55) = happyShift action_25
action_239 (69) = happyShift action_26
action_239 (35) = happyGoto action_255
action_239 _ = happyFail

action_240 (44) = happyShift action_23
action_240 (53) = happyShift action_24
action_240 (55) = happyShift action_25
action_240 (69) = happyShift action_26
action_240 (33) = happyGoto action_254
action_240 (35) = happyGoto action_213
action_240 _ = happyFail

action_241 (40) = happyShift action_253
action_241 _ = happyFail

action_242 (45) = happyShift action_79
action_242 (47) = happyShift action_252
action_242 (49) = happyShift action_80
action_242 (50) = happyShift action_81
action_242 (52) = happyShift action_82
action_242 (54) = happyShift action_83
action_242 (55) = happyShift action_84
action_242 (56) = happyShift action_85
action_242 (57) = happyShift action_86
action_242 (60) = happyShift action_87
action_242 (61) = happyShift action_88
action_242 (63) = happyShift action_89
action_242 (65) = happyShift action_90
action_242 (66) = happyShift action_91
action_242 _ = happyFail

action_243 (48) = happyShift action_251
action_243 _ = happyFail

action_244 (45) = happyShift action_79
action_244 (47) = happyShift action_250
action_244 (49) = happyShift action_80
action_244 (50) = happyShift action_81
action_244 (52) = happyShift action_82
action_244 (54) = happyShift action_83
action_244 (55) = happyShift action_84
action_244 (56) = happyShift action_85
action_244 (57) = happyShift action_86
action_244 (60) = happyShift action_87
action_244 (61) = happyShift action_88
action_244 (63) = happyShift action_89
action_244 (65) = happyShift action_90
action_244 (66) = happyShift action_91
action_244 _ = happyFail

action_245 (45) = happyShift action_79
action_245 (47) = happyShift action_249
action_245 (49) = happyShift action_80
action_245 (50) = happyShift action_81
action_245 (52) = happyShift action_82
action_245 (54) = happyShift action_83
action_245 (55) = happyShift action_84
action_245 (56) = happyShift action_85
action_245 (57) = happyShift action_86
action_245 (60) = happyShift action_87
action_245 (61) = happyShift action_88
action_245 (63) = happyShift action_89
action_245 (65) = happyShift action_90
action_245 (66) = happyShift action_91
action_245 _ = happyFail

action_246 _ = happyReduce_32

action_247 _ = happyReduce_33

action_248 _ = happyReduce_35

action_249 _ = happyReduce_14

action_250 _ = happyReduce_71

action_251 _ = happyReduce_38

action_252 _ = happyReduce_30

action_253 (37) = happyShift action_60
action_253 (44) = happyShift action_61
action_253 (53) = happyShift action_24
action_253 (55) = happyShift action_25
action_253 (60) = happyShift action_62
action_253 (69) = happyShift action_63
action_253 (21) = happyGoto action_268
action_253 (22) = happyGoto action_58
action_253 (35) = happyGoto action_59
action_253 _ = happyFail

action_254 _ = happyReduce_85

action_255 (45) = happyShift action_79
action_255 (49) = happyShift action_80
action_255 (50) = happyShift action_81
action_255 (52) = happyShift action_82
action_255 (54) = happyShift action_83
action_255 (55) = happyShift action_84
action_255 (56) = happyShift action_85
action_255 (57) = happyShift action_86
action_255 (60) = happyShift action_87
action_255 (61) = happyShift action_88
action_255 (63) = happyShift action_89
action_255 (65) = happyShift action_90
action_255 (66) = happyShift action_91
action_255 _ = happyReduce_86

action_256 (58) = happyShift action_267
action_256 _ = happyFail

action_257 (41) = happyShift action_266
action_257 _ = happyReduce_72

action_258 (45) = happyShift action_79
action_258 (49) = happyShift action_80
action_258 (50) = happyShift action_81
action_258 (52) = happyShift action_82
action_258 (54) = happyShift action_83
action_258 (55) = happyShift action_84
action_258 (56) = happyShift action_85
action_258 (57) = happyShift action_86
action_258 (60) = happyShift action_87
action_258 (61) = happyShift action_88
action_258 (63) = happyShift action_89
action_258 (65) = happyShift action_90
action_258 (66) = happyShift action_91
action_258 (68) = happyShift action_265
action_258 _ = happyFail

action_259 (48) = happyShift action_264
action_259 _ = happyFail

action_260 (44) = happyShift action_75
action_260 (58) = happyShift action_263
action_260 _ = happyReduce_105

action_261 (37) = happyShift action_60
action_261 (44) = happyShift action_61
action_261 (53) = happyShift action_24
action_261 (55) = happyShift action_25
action_261 (60) = happyShift action_62
action_261 (69) = happyShift action_63
action_261 (21) = happyGoto action_262
action_261 (22) = happyGoto action_58
action_261 (35) = happyGoto action_59
action_261 _ = happyFail

action_262 (48) = happyShift action_274
action_262 (51) = happyShift action_151
action_262 _ = happyFail

action_263 (46) = happyShift action_113
action_263 (69) = happyShift action_114
action_263 (23) = happyGoto action_273
action_263 _ = happyFail

action_264 _ = happyReduce_61

action_265 (37) = happyShift action_60
action_265 (44) = happyShift action_61
action_265 (53) = happyShift action_24
action_265 (55) = happyShift action_25
action_265 (60) = happyShift action_62
action_265 (69) = happyShift action_63
action_265 (21) = happyGoto action_272
action_265 (22) = happyGoto action_58
action_265 (35) = happyGoto action_59
action_265 _ = happyFail

action_266 (69) = happyShift action_271
action_266 (24) = happyGoto action_270
action_266 (25) = happyGoto action_257
action_266 _ = happyFail

action_267 (37) = happyShift action_60
action_267 (44) = happyShift action_61
action_267 (53) = happyShift action_24
action_267 (55) = happyShift action_25
action_267 (60) = happyShift action_62
action_267 (69) = happyShift action_63
action_267 (21) = happyGoto action_269
action_267 (22) = happyGoto action_58
action_267 (35) = happyGoto action_59
action_267 _ = happyFail

action_268 _ = happyReduce_59

action_269 (48) = happyShift action_277
action_269 (51) = happyShift action_151
action_269 _ = happyFail

action_270 _ = happyReduce_73

action_271 (58) = happyShift action_263
action_271 _ = happyFail

action_272 (51) = happyShift action_276
action_272 _ = happyReduce_107

action_273 (39) = happyShift action_275
action_273 _ = happyFail

action_274 _ = happyReduce_62

action_275 (44) = happyShift action_23
action_275 (53) = happyShift action_24
action_275 (55) = happyShift action_25
action_275 (69) = happyShift action_26
action_275 (35) = happyGoto action_280
action_275 _ = happyFail

action_276 (37) = happyShift action_60
action_276 (44) = happyShift action_61
action_276 (53) = happyShift action_24
action_276 (55) = happyShift action_25
action_276 (60) = happyShift action_62
action_276 (69) = happyShift action_63
action_276 (21) = happyGoto action_170
action_276 (22) = happyGoto action_58
action_276 (35) = happyGoto action_278
action_276 (36) = happyGoto action_279
action_276 _ = happyFail

action_277 _ = happyReduce_60

action_278 (45) = happyShift action_79
action_278 (49) = happyShift action_80
action_278 (50) = happyShift action_81
action_278 (52) = happyShift action_82
action_278 (54) = happyShift action_83
action_278 (55) = happyShift action_84
action_278 (56) = happyShift action_85
action_278 (57) = happyShift action_86
action_278 (60) = happyShift action_87
action_278 (61) = happyShift action_88
action_278 (63) = happyShift action_89
action_278 (65) = happyShift action_90
action_278 (66) = happyShift action_91
action_278 (67) = happyShift action_150
action_278 (68) = happyShift action_265
action_278 _ = happyFail

action_279 _ = happyReduce_108

action_280 (45) = happyShift action_79
action_280 (49) = happyShift action_80
action_280 (50) = happyShift action_81
action_280 (52) = happyShift action_82
action_280 (54) = happyShift action_83
action_280 (55) = happyShift action_84
action_280 (56) = happyShift action_85
action_280 (57) = happyShift action_86
action_280 (60) = happyShift action_87
action_280 (61) = happyShift action_88
action_280 (63) = happyShift action_89
action_280 (65) = happyShift action_90
action_280 (66) = happyShift action_91
action_280 _ = happyReduce_74

happyReduce_1 = happySpecReduce_1  4 happyReduction_1
happyReduction_1 (HappyAbsSyn5  happy_var_1)
	 =  HappyAbsSyn4
		 (happy_var_1
	)
happyReduction_1 _  = notHappyAtAll 

happyReduce_2 = happySpecReduce_0  5 happyReduction_2
happyReduction_2  =  HappyAbsSyn5
		 ([]
	)

happyReduce_3 = happySpecReduce_2  5 happyReduction_3
happyReduction_3 (HappyAbsSyn5  happy_var_2)
	(HappyAbsSyn6  happy_var_1)
	 =  HappyAbsSyn5
		 (happy_var_1 : happy_var_2
	)
happyReduction_3 _ _  = notHappyAtAll 

happyReduce_4 = happySpecReduce_2  6 happyReduction_4
happyReduction_4 (HappyAbsSyn17  happy_var_2)
	_
	 =  HappyAbsSyn6
		 (DataHiding happy_var_2
	)
happyReduction_4 _ _  = notHappyAtAll 

happyReduce_5 = happySpecReduce_2  6 happyReduction_5
happyReduction_5 (HappyAbsSyn15  happy_var_2)
	_
	 =  HappyAbsSyn6
		 (DataRenaming happy_var_2
	)
happyReduction_5 _ _  = notHappyAtAll 

happyReduce_6 = happySpecReduce_2  6 happyReduction_6
happyReduction_6 (HappyAbsSyn8  happy_var_2)
	_
	 =  HappyAbsSyn6
		 (DataActions happy_var_2
	)
happyReduction_6 _ _  = notHappyAtAll 

happyReduce_7 = happyReduce 6 6 happyReduction_7
happyReduction_7 ((HappyAbsSyn35  happy_var_6) `HappyStk`
	_ `HappyStk`
	(HappyAbsSyn23  happy_var_4) `HappyStk`
	_ `HappyStk`
	(HappyTerminal (TokenString happy_var_2)) `HappyStk`
	_ `HappyStk`
	happyRest)
	 = HappyAbsSyn6
		 (DataGlobal (happy_var_2, happy_var_4) happy_var_6
	) `HappyStk` happyRest

happyReduce_8 = happySpecReduce_2  6 happyReduction_8
happyReduction_8 (HappyAbsSyn17  happy_var_2)
	_
	 =  HappyAbsSyn6
		 (DataEncapsulation happy_var_2
	)
happyReduction_8 _ _  = notHappyAtAll 

happyReduce_9 = happySpecReduce_2  6 happyReduction_9
happyReduction_9 (HappyAbsSyn13  happy_var_2)
	_
	 =  HappyAbsSyn6
		 (DataCommunication happy_var_2
	)
happyReduction_9 _ _  = notHappyAtAll 

happyReduce_10 = happyReduce 4 6 happyReduction_10
happyReduction_10 (_ `HappyStk`
	(HappyAbsSyn7  happy_var_3) `HappyStk`
	_ `HappyStk`
	_ `HappyStk`
	happyRest)
	 = HappyAbsSyn6
		 (DataUntilFormula happy_var_3
	) `HappyStk` happyRest

happyReduce_11 = happySpecReduce_2  6 happyReduction_11
happyReduction_11 (HappyAbsSyn35  happy_var_2)
	_
	 =  HappyAbsSyn6
		 (DataFormula happy_var_2
	)
happyReduction_11 _ _  = notHappyAtAll 

happyReduce_12 = happyReduce 4 6 happyReduction_12
happyReduction_12 ((HappyAbsSyn35  happy_var_4) `HappyStk`
	_ `HappyStk`
	(HappyTerminal (TokenString happy_var_2)) `HappyStk`
	_ `HappyStk`
	happyRest)
	 = HappyAbsSyn6
		 (DataConstant happy_var_2 happy_var_4
	) `HappyStk` happyRest

happyReduce_13 = happyReduce 6 6 happyReduction_13
happyReduction_13 (_ `HappyStk`
	(HappyAbsSyn17  happy_var_5) `HappyStk`
	_ `HappyStk`
	_ `HappyStk`
	(HappyTerminal (TokenString happy_var_2)) `HappyStk`
	_ `HappyStk`
	happyRest)
	 = HappyAbsSyn6
		 (DataEnumType happy_var_2 happy_var_5
	) `HappyStk` happyRest

happyReduce_14 = happyReduce 8 6 happyReduction_14
happyReduction_14 (_ `HappyStk`
	(HappyAbsSyn35  happy_var_7) `HappyStk`
	_ `HappyStk`
	(HappyAbsSyn35  happy_var_5) `HappyStk`
	_ `HappyStk`
	_ `HappyStk`
	(HappyTerminal (TokenString happy_var_2)) `HappyStk`
	_ `HappyStk`
	happyRest)
	 = HappyAbsSyn6
		 (DataRangeType happy_var_2 happy_var_5 happy_var_7
	) `HappyStk` happyRest

happyReduce_15 = happyReduce 6 6 happyReduction_15
happyReduction_15 (_ `HappyStk`
	(HappyAbsSyn10  happy_var_5) `HappyStk`
	_ `HappyStk`
	_ `HappyStk`
	(HappyTerminal (TokenString happy_var_2)) `HappyStk`
	_ `HappyStk`
	happyRest)
	 = HappyAbsSyn6
		 (DataFunction happy_var_2 happy_var_5
	) `HappyStk` happyRest

happyReduce_16 = happySpecReduce_2  6 happyReduction_16
happyReduction_16 (HappyAbsSyn18  happy_var_2)
	_
	 =  HappyAbsSyn6
		 (ParserInitialProcess happy_var_2
	)
happyReduction_16 _ _  = notHappyAtAll 

happyReduce_17 = happyReduce 6 6 happyReduction_17
happyReduction_17 ((HappyAbsSyn21  happy_var_6) `HappyStk`
	_ `HappyStk`
	_ `HappyStk`
	(HappyAbsSyn26  happy_var_3) `HappyStk`
	_ `HappyStk`
	(HappyTerminal (TokenString happy_var_1)) `HappyStk`
	happyRest)
	 = HappyAbsSyn6
		 (ParserProcess (Process happy_var_1 (reverse happy_var_3) happy_var_6)
	) `HappyStk` happyRest

happyReduce_18 = happySpecReduce_3  6 happyReduction_18
happyReduction_18 (HappyAbsSyn21  happy_var_3)
	_
	(HappyTerminal (TokenString happy_var_1))
	 =  HappyAbsSyn6
		 (ParserProcess (Process happy_var_1 [] happy_var_3)
	)
happyReduction_18 _ _ _  = notHappyAtAll 

happyReduce_19 = happySpecReduce_0  7 happyReduction_19
happyReduction_19  =  HappyAbsSyn7
		 (""
	)

happyReduce_20 = happySpecReduce_2  7 happyReduction_20
happyReduction_20 (HappyAbsSyn7  happy_var_2)
	(HappyTerminal (TokenString happy_var_1))
	 =  HappyAbsSyn7
		 (happy_var_1 ++ " " ++ happy_var_2
	)
happyReduction_20 _ _  = notHappyAtAll 

happyReduce_21 = happySpecReduce_2  7 happyReduction_21
happyReduction_21 (HappyAbsSyn7  happy_var_2)
	_
	 =  HappyAbsSyn7
		 ("|" ++ " " ++ happy_var_2
	)
happyReduction_21 _ _  = notHappyAtAll 

happyReduce_22 = happySpecReduce_2  7 happyReduction_22
happyReduction_22 (HappyAbsSyn7  happy_var_2)
	_
	 =  HappyAbsSyn7
		 ("!" ++ " " ++ happy_var_2
	)
happyReduction_22 _ _  = notHappyAtAll 

happyReduce_23 = happySpecReduce_2  7 happyReduction_23
happyReduction_23 (HappyAbsSyn7  happy_var_2)
	_
	 =  HappyAbsSyn7
		 ("(" ++ " " ++ happy_var_2
	)
happyReduction_23 _ _  = notHappyAtAll 

happyReduce_24 = happySpecReduce_2  7 happyReduction_24
happyReduction_24 (HappyAbsSyn7  happy_var_2)
	_
	 =  HappyAbsSyn7
		 (")" ++ " " ++ happy_var_2
	)
happyReduction_24 _ _  = notHappyAtAll 

happyReduce_25 = happySpecReduce_2  7 happyReduction_25
happyReduction_25 (HappyAbsSyn7  happy_var_2)
	_
	 =  HappyAbsSyn7
		 ("," ++ " " ++ happy_var_2
	)
happyReduction_25 _ _  = notHappyAtAll 

happyReduce_26 = happySpecReduce_1  8 happyReduction_26
happyReduction_26 (HappyAbsSyn9  happy_var_1)
	 =  HappyAbsSyn8
		 ([happy_var_1]
	)
happyReduction_26 _  = notHappyAtAll 

happyReduce_27 = happySpecReduce_3  8 happyReduction_27
happyReduction_27 (HappyAbsSyn8  happy_var_3)
	_
	(HappyAbsSyn9  happy_var_1)
	 =  HappyAbsSyn8
		 (happy_var_1:happy_var_3
	)
happyReduction_27 _ _ _  = notHappyAtAll 

happyReduce_28 = happySpecReduce_1  9 happyReduction_28
happyReduction_28 (HappyTerminal (TokenString happy_var_1))
	 =  HappyAbsSyn9
		 ((happy_var_1, None)
	)
happyReduction_28 _  = notHappyAtAll 

happyReduce_29 = happySpecReduce_3  9 happyReduction_29
happyReduction_29 _
	_
	(HappyTerminal (TokenString happy_var_1))
	 =  HappyAbsSyn9
		 ((happy_var_1, Bool)
	)
happyReduction_29 _ _ _  = notHappyAtAll 

happyReduce_30 = happyReduce 7 9 happyReduction_30
happyReduction_30 (_ `HappyStk`
	(HappyAbsSyn35  happy_var_6) `HappyStk`
	_ `HappyStk`
	(HappyAbsSyn35  happy_var_4) `HappyStk`
	_ `HappyStk`
	_ `HappyStk`
	(HappyTerminal (TokenString happy_var_1)) `HappyStk`
	happyRest)
	 = HappyAbsSyn9
		 ((happy_var_1, IntRange happy_var_4 happy_var_6)
	) `HappyStk` happyRest

happyReduce_31 = happySpecReduce_1  10 happyReduction_31
happyReduction_31 (HappyAbsSyn11  happy_var_1)
	 =  HappyAbsSyn10
		 ([happy_var_1]
	)
happyReduction_31 _  = notHappyAtAll 

happyReduce_32 = happySpecReduce_3  10 happyReduction_32
happyReduction_32 (HappyAbsSyn10  happy_var_3)
	_
	(HappyAbsSyn11  happy_var_1)
	 =  HappyAbsSyn10
		 (happy_var_1:happy_var_3
	)
happyReduction_32 _ _ _  = notHappyAtAll 

happyReduce_33 = happySpecReduce_3  11 happyReduction_33
happyReduction_33 (HappyTerminal (TokenString happy_var_3))
	_
	(HappyAbsSyn12  happy_var_1)
	 =  HappyAbsSyn11
		 ((happy_var_1, happy_var_3)
	)
happyReduction_33 _ _ _  = notHappyAtAll 

happyReduce_34 = happySpecReduce_1  12 happyReduction_34
happyReduction_34 (HappyTerminal (TokenString happy_var_1))
	 =  HappyAbsSyn12
		 ([happy_var_1]
	)
happyReduction_34 _  = notHappyAtAll 

happyReduce_35 = happySpecReduce_3  12 happyReduction_35
happyReduction_35 (HappyAbsSyn12  happy_var_3)
	_
	(HappyTerminal (TokenString happy_var_1))
	 =  HappyAbsSyn12
		 (happy_var_1:happy_var_3
	)
happyReduction_35 _ _ _  = notHappyAtAll 

happyReduce_36 = happySpecReduce_1  13 happyReduction_36
happyReduction_36 (HappyAbsSyn14  happy_var_1)
	 =  HappyAbsSyn13
		 ([happy_var_1]
	)
happyReduction_36 _  = notHappyAtAll 

happyReduce_37 = happySpecReduce_3  13 happyReduction_37
happyReduction_37 (HappyAbsSyn13  happy_var_3)
	_
	(HappyAbsSyn14  happy_var_1)
	 =  HappyAbsSyn13
		 (happy_var_1 : happy_var_3
	)
happyReduction_37 _ _ _  = notHappyAtAll 

happyReduce_38 = happyReduce 7 14 happyReduction_38
happyReduction_38 (_ `HappyStk`
	(HappyTerminal (TokenString happy_var_6)) `HappyStk`
	_ `HappyStk`
	(HappyTerminal (TokenString happy_var_4)) `HappyStk`
	_ `HappyStk`
	(HappyTerminal (TokenString happy_var_2)) `HappyStk`
	_ `HappyStk`
	happyRest)
	 = HappyAbsSyn14
		 ((happy_var_2,happy_var_4,happy_var_6)
	) `HappyStk` happyRest

happyReduce_39 = happySpecReduce_1  15 happyReduction_39
happyReduction_39 (HappyAbsSyn16  happy_var_1)
	 =  HappyAbsSyn15
		 ([happy_var_1]
	)
happyReduction_39 _  = notHappyAtAll 

happyReduce_40 = happySpecReduce_3  15 happyReduction_40
happyReduction_40 (HappyAbsSyn15  happy_var_3)
	_
	(HappyAbsSyn16  happy_var_1)
	 =  HappyAbsSyn15
		 (happy_var_1 : happy_var_3
	)
happyReduction_40 _ _ _  = notHappyAtAll 

happyReduce_41 = happyReduce 5 16 happyReduction_41
happyReduction_41 (_ `HappyStk`
	(HappyTerminal (TokenString happy_var_4)) `HappyStk`
	_ `HappyStk`
	(HappyTerminal (TokenString happy_var_2)) `HappyStk`
	_ `HappyStk`
	happyRest)
	 = HappyAbsSyn16
		 ((happy_var_2,happy_var_4)
	) `HappyStk` happyRest

happyReduce_42 = happySpecReduce_1  17 happyReduction_42
happyReduction_42 (HappyTerminal (TokenString happy_var_1))
	 =  HappyAbsSyn17
		 ([happy_var_1]
	)
happyReduction_42 _  = notHappyAtAll 

happyReduce_43 = happySpecReduce_3  17 happyReduction_43
happyReduction_43 (HappyAbsSyn17  happy_var_3)
	_
	(HappyTerminal (TokenString happy_var_1))
	 =  HappyAbsSyn17
		 (happy_var_1 : happy_var_3
	)
happyReduction_43 _ _ _  = notHappyAtAll 

happyReduce_44 = happySpecReduce_2  18 happyReduction_44
happyReduction_44 (HappyAbsSyn20  happy_var_2)
	(HappyTerminal (TokenString happy_var_1))
	 =  HappyAbsSyn18
		 (InitSingleProcess (InitialProcess happy_var_1 happy_var_2 [])
	)
happyReduction_44 _ _  = notHappyAtAll 

happyReduce_45 = happyReduce 5 18 happyReduction_45
happyReduction_45 ((HappyAbsSyn20  happy_var_5) `HappyStk`
	_ `HappyStk`
	(HappyAbsSyn30  happy_var_3) `HappyStk`
	_ `HappyStk`
	(HappyTerminal (TokenString happy_var_1)) `HappyStk`
	happyRest)
	 = HappyAbsSyn18
		 (InitSingleProcess (InitialProcess happy_var_1 happy_var_5 happy_var_3)
	) `HappyStk` happyRest

happyReduce_46 = happySpecReduce_3  18 happyReduction_46
happyReduction_46 (HappyAbsSyn18  happy_var_3)
	_
	(HappyAbsSyn18  happy_var_1)
	 =  HappyAbsSyn18
		 (InitParallel happy_var_1 happy_var_3
	)
happyReduction_46 _ _ _  = notHappyAtAll 

happyReduce_47 = happyReduce 6 18 happyReduction_47
happyReduction_47 (_ `HappyStk`
	(HappyAbsSyn18  happy_var_5) `HappyStk`
	_ `HappyStk`
	(HappyAbsSyn17  happy_var_3) `HappyStk`
	_ `HappyStk`
	_ `HappyStk`
	happyRest)
	 = HappyAbsSyn18
		 (InitHiding happy_var_3 happy_var_5
	) `HappyStk` happyRest

happyReduce_48 = happyReduce 6 18 happyReduction_48
happyReduction_48 (_ `HappyStk`
	(HappyAbsSyn18  happy_var_5) `HappyStk`
	_ `HappyStk`
	(HappyAbsSyn17  happy_var_3) `HappyStk`
	_ `HappyStk`
	_ `HappyStk`
	happyRest)
	 = HappyAbsSyn18
		 (InitEncapsulation happy_var_3 happy_var_5
	) `HappyStk` happyRest

happyReduce_49 = happyReduce 6 18 happyReduction_49
happyReduction_49 (_ `HappyStk`
	(HappyAbsSyn18  happy_var_5) `HappyStk`
	_ `HappyStk`
	(HappyAbsSyn15  happy_var_3) `HappyStk`
	_ `HappyStk`
	_ `HappyStk`
	happyRest)
	 = HappyAbsSyn18
		 (InitRenaming happy_var_3 happy_var_5
	) `HappyStk` happyRest

happyReduce_50 = happySpecReduce_3  18 happyReduction_50
happyReduction_50 _
	(HappyAbsSyn18  happy_var_2)
	_
	 =  HappyAbsSyn18
		 (happy_var_2
	)
happyReduction_50 _ _ _  = notHappyAtAll 

happyReduce_51 = happySpecReduce_0  19 happyReduction_51
happyReduction_51  =  HappyAbsSyn19
		 ([]
	)

happyReduce_52 = happySpecReduce_2  19 happyReduction_52
happyReduction_52 _
	_
	 =  HappyAbsSyn19
		 ([]
	)

happyReduce_53 = happySpecReduce_3  19 happyReduction_53
happyReduction_53 _
	(HappyAbsSyn34  happy_var_2)
	_
	 =  HappyAbsSyn19
		 (reverse happy_var_2
	)
happyReduction_53 _ _ _  = notHappyAtAll 

happyReduce_54 = happySpecReduce_0  20 happyReduction_54
happyReduction_54  =  HappyAbsSyn20
		 ([]
	)

happyReduce_55 = happySpecReduce_2  20 happyReduction_55
happyReduction_55 _
	_
	 =  HappyAbsSyn20
		 ([]
	)

happyReduce_56 = happySpecReduce_3  20 happyReduction_56
happyReduction_56 _
	(HappyAbsSyn34  happy_var_2)
	_
	 =  HappyAbsSyn20
		 (reverse happy_var_2
	)
happyReduction_56 _ _ _  = notHappyAtAll 

happyReduce_57 = happySpecReduce_3  21 happyReduction_57
happyReduction_57 (HappyAbsSyn21  happy_var_3)
	_
	(HappyAbsSyn21  happy_var_1)
	 =  HappyAbsSyn21
		 (Plus ([happy_var_1] ++ [happy_var_3])
	)
happyReduction_57 _ _ _  = notHappyAtAll 

happyReduce_58 = happyReduce 4 21 happyReduction_58
happyReduction_58 ((HappyAbsSyn21  happy_var_4) `HappyStk`
	_ `HappyStk`
	(HappyAbsSyn19  happy_var_2) `HappyStk`
	(HappyTerminal (TokenString happy_var_1)) `HappyStk`
	happyRest)
	 = HappyAbsSyn21
		 (ActionPrefix happy_var_1 happy_var_2 [("i0", TypeRange 1 1, (Variable "1"))] happy_var_4
	) `HappyStk` happyRest

happyReduce_59 = happyReduce 7 21 happyReduction_59
happyReduction_59 ((HappyAbsSyn21  happy_var_7) `HappyStk`
	_ `HappyStk`
	_ `HappyStk`
	(HappyAbsSyn32  happy_var_4) `HappyStk`
	_ `HappyStk`
	(HappyAbsSyn19  happy_var_2) `HappyStk`
	(HappyTerminal (TokenString happy_var_1)) `HappyStk`
	happyRest)
	 = HappyAbsSyn21
		 (ActionPrefix (happy_var_1 ++ "{" ++ (printExpressions happy_var_4) ++ "}") happy_var_2 [("i0", TypeRange 1 1, (Variable "1"))] happy_var_7
	) `HappyStk` happyRest

happyReduce_60 = happyReduce 9 21 happyReduction_60
happyReduction_60 (_ `HappyStk`
	(HappyAbsSyn21  happy_var_8) `HappyStk`
	_ `HappyStk`
	(HappyAbsSyn24  happy_var_6) `HappyStk`
	_ `HappyStk`
	_ `HappyStk`
	_ `HappyStk`
	(HappyAbsSyn19  happy_var_2) `HappyStk`
	(HappyTerminal (TokenString happy_var_1)) `HappyStk`
	happyRest)
	 = HappyAbsSyn21
		 (ActionPrefix happy_var_1 happy_var_2 happy_var_6 happy_var_8
	) `HappyStk` happyRest

happyReduce_61 = happyReduce 7 21 happyReduction_61
happyReduction_61 (_ `HappyStk`
	(HappyAbsSyn36  happy_var_6) `HappyStk`
	_ `HappyStk`
	_ `HappyStk`
	_ `HappyStk`
	(HappyAbsSyn19  happy_var_2) `HappyStk`
	(HappyTerminal (TokenString happy_var_1)) `HappyStk`
	happyRest)
	 = HappyAbsSyn21
		 (ActionPrefix2 happy_var_1 happy_var_2 happy_var_6
	) `HappyStk` happyRest

happyReduce_62 = happyReduce 8 21 happyReduction_62
happyReduction_62 (_ `HappyStk`
	(HappyAbsSyn21  happy_var_7) `HappyStk`
	_ `HappyStk`
	(HappyAbsSyn23  happy_var_5) `HappyStk`
	_ `HappyStk`
	(HappyTerminal (TokenString happy_var_3)) `HappyStk`
	_ `HappyStk`
	_ `HappyStk`
	happyRest)
	 = HappyAbsSyn21
		 (Sum happy_var_3 happy_var_5 happy_var_7
	) `HappyStk` happyRest

happyReduce_63 = happySpecReduce_3  21 happyReduction_63
happyReduction_63 (HappyAbsSyn21  happy_var_3)
	_
	(HappyAbsSyn35  happy_var_1)
	 =  HappyAbsSyn21
		 (Implication happy_var_1 happy_var_3
	)
happyReduction_63 _ _ _  = notHappyAtAll 

happyReduce_64 = happySpecReduce_1  21 happyReduction_64
happyReduction_64 (HappyAbsSyn22  happy_var_1)
	 =  HappyAbsSyn21
		 (happy_var_1
	)
happyReduction_64 _  = notHappyAtAll 

happyReduce_65 = happySpecReduce_3  21 happyReduction_65
happyReduction_65 _
	(HappyAbsSyn21  happy_var_2)
	_
	 =  HappyAbsSyn21
		 (happy_var_2
	)
happyReduction_65 _ _ _  = notHappyAtAll 

happyReduce_66 = happyReduce 5 21 happyReduction_66
happyReduction_66 ((HappyAbsSyn21  happy_var_5) `HappyStk`
	_ `HappyStk`
	_ `HappyStk`
	(HappyAbsSyn35  happy_var_2) `HappyStk`
	_ `HappyStk`
	happyRest)
	 = HappyAbsSyn21
		 (LambdaPrefix happy_var_2 happy_var_5
	) `HappyStk` happyRest

happyReduce_67 = happyReduce 4 22 happyReduction_67
happyReduction_67 (_ `HappyStk`
	(HappyAbsSyn34  happy_var_3) `HappyStk`
	_ `HappyStk`
	(HappyTerminal (TokenString happy_var_1)) `HappyStk`
	happyRest)
	 = HappyAbsSyn22
		 (ProcessInstance happy_var_1 (reverse happy_var_3)
	) `HappyStk` happyRest

happyReduce_68 = happyReduce 4 22 happyReduction_68
happyReduction_68 (_ `HappyStk`
	(HappyAbsSyn28  happy_var_3) `HappyStk`
	_ `HappyStk`
	(HappyTerminal (TokenString happy_var_1)) `HappyStk`
	happyRest)
	 = HappyAbsSyn22
		 (ProcessInstance2 happy_var_1 (reverse happy_var_3)
	) `HappyStk` happyRest

happyReduce_69 = happySpecReduce_3  22 happyReduction_69
happyReduction_69 _
	_
	(HappyTerminal (TokenString happy_var_1))
	 =  HappyAbsSyn22
		 (ProcessInstance2 happy_var_1 []
	)
happyReduction_69 _ _ _  = notHappyAtAll 

happyReduce_70 = happySpecReduce_1  23 happyReduction_70
happyReduction_70 (HappyTerminal (TokenString happy_var_1))
	 =  HappyAbsSyn23
		 (TypeName happy_var_1
	)
happyReduction_70 _  = notHappyAtAll 

happyReduce_71 = happyReduce 5 23 happyReduction_71
happyReduction_71 (_ `HappyStk`
	(HappyAbsSyn35  happy_var_4) `HappyStk`
	_ `HappyStk`
	(HappyAbsSyn35  happy_var_2) `HappyStk`
	_ `HappyStk`
	happyRest)
	 = HappyAbsSyn23
		 (TypeRangeExpressions happy_var_2 happy_var_4
	) `HappyStk` happyRest

happyReduce_72 = happySpecReduce_1  24 happyReduction_72
happyReduction_72 (HappyAbsSyn25  happy_var_1)
	 =  HappyAbsSyn24
		 ([happy_var_1]
	)
happyReduction_72 _  = notHappyAtAll 

happyReduce_73 = happySpecReduce_3  24 happyReduction_73
happyReduction_73 (HappyAbsSyn24  happy_var_3)
	_
	(HappyAbsSyn25  happy_var_1)
	 =  HappyAbsSyn24
		 (happy_var_1 : happy_var_3
	)
happyReduction_73 _ _ _  = notHappyAtAll 

happyReduce_74 = happyReduce 5 25 happyReduction_74
happyReduction_74 ((HappyAbsSyn35  happy_var_5) `HappyStk`
	_ `HappyStk`
	(HappyAbsSyn23  happy_var_3) `HappyStk`
	_ `HappyStk`
	(HappyTerminal (TokenString happy_var_1)) `HappyStk`
	happyRest)
	 = HappyAbsSyn25
		 ((happy_var_1, happy_var_3, happy_var_5)
	) `HappyStk` happyRest

happyReduce_75 = happySpecReduce_1  26 happyReduction_75
happyReduction_75 (HappyAbsSyn27  happy_var_1)
	 =  HappyAbsSyn26
		 ([happy_var_1]
	)
happyReduction_75 _  = notHappyAtAll 

happyReduce_76 = happySpecReduce_3  26 happyReduction_76
happyReduction_76 (HappyAbsSyn27  happy_var_3)
	_
	(HappyAbsSyn26  happy_var_1)
	 =  HappyAbsSyn26
		 (happy_var_3 : happy_var_1
	)
happyReduction_76 _ _ _  = notHappyAtAll 

happyReduce_77 = happySpecReduce_3  27 happyReduction_77
happyReduction_77 (HappyAbsSyn23  happy_var_3)
	_
	(HappyTerminal (TokenString happy_var_1))
	 =  HappyAbsSyn27
		 ((happy_var_1, happy_var_3)
	)
happyReduction_77 _ _ _  = notHappyAtAll 

happyReduce_78 = happySpecReduce_1  28 happyReduction_78
happyReduction_78 (HappyAbsSyn29  happy_var_1)
	 =  HappyAbsSyn28
		 ([happy_var_1]
	)
happyReduction_78 _  = notHappyAtAll 

happyReduce_79 = happySpecReduce_3  28 happyReduction_79
happyReduction_79 (HappyAbsSyn29  happy_var_3)
	_
	(HappyAbsSyn28  happy_var_1)
	 =  HappyAbsSyn28
		 (happy_var_3 : happy_var_1
	)
happyReduction_79 _ _ _  = notHappyAtAll 

happyReduce_80 = happySpecReduce_3  29 happyReduction_80
happyReduction_80 (HappyAbsSyn35  happy_var_3)
	_
	(HappyTerminal (TokenString happy_var_1))
	 =  HappyAbsSyn29
		 ((happy_var_1, happy_var_3)
	)
happyReduction_80 _ _ _  = notHappyAtAll 

happyReduce_81 = happySpecReduce_1  30 happyReduction_81
happyReduction_81 (HappyAbsSyn31  happy_var_1)
	 =  HappyAbsSyn30
		 ([happy_var_1]
	)
happyReduction_81 _  = notHappyAtAll 

happyReduce_82 = happySpecReduce_3  30 happyReduction_82
happyReduction_82 (HappyAbsSyn31  happy_var_3)
	_
	(HappyAbsSyn30  happy_var_1)
	 =  HappyAbsSyn30
		 (happy_var_3 : happy_var_1
	)
happyReduction_82 _ _ _  = notHappyAtAll 

happyReduce_83 = happySpecReduce_3  31 happyReduction_83
happyReduction_83 (HappyTerminal (TokenString happy_var_3))
	_
	(HappyTerminal (TokenString happy_var_1))
	 =  HappyAbsSyn31
		 ((happy_var_1, happy_var_3)
	)
happyReduction_83 _ _ _  = notHappyAtAll 

happyReduce_84 = happySpecReduce_1  32 happyReduction_84
happyReduction_84 (HappyAbsSyn33  happy_var_1)
	 =  HappyAbsSyn32
		 ([happy_var_1]
	)
happyReduction_84 _  = notHappyAtAll 

happyReduce_85 = happySpecReduce_3  32 happyReduction_85
happyReduction_85 (HappyAbsSyn33  happy_var_3)
	_
	(HappyAbsSyn32  happy_var_1)
	 =  HappyAbsSyn32
		 (happy_var_3 : happy_var_1
	)
happyReduction_85 _ _ _  = notHappyAtAll 

happyReduce_86 = happySpecReduce_3  33 happyReduction_86
happyReduction_86 (HappyAbsSyn35  happy_var_3)
	_
	(HappyAbsSyn35  happy_var_1)
	 =  HappyAbsSyn33
		 ((happy_var_1, happy_var_3)
	)
happyReduction_86 _ _ _  = notHappyAtAll 

happyReduce_87 = happySpecReduce_1  34 happyReduction_87
happyReduction_87 (HappyAbsSyn35  happy_var_1)
	 =  HappyAbsSyn34
		 ([happy_var_1]
	)
happyReduction_87 _  = notHappyAtAll 

happyReduce_88 = happySpecReduce_3  34 happyReduction_88
happyReduction_88 (HappyAbsSyn35  happy_var_3)
	_
	(HappyAbsSyn34  happy_var_1)
	 =  HappyAbsSyn34
		 (happy_var_3 : happy_var_1
	)
happyReduction_88 _ _ _  = notHappyAtAll 

happyReduce_89 = happySpecReduce_3  35 happyReduction_89
happyReduction_89 (HappyAbsSyn35  happy_var_3)
	_
	(HappyAbsSyn35  happy_var_1)
	 =  HappyAbsSyn35
		 (Function "lt" [happy_var_1, happy_var_3]
	)
happyReduction_89 _ _ _  = notHappyAtAll 

happyReduce_90 = happySpecReduce_3  35 happyReduction_90
happyReduction_90 (HappyAbsSyn35  happy_var_3)
	_
	(HappyAbsSyn35  happy_var_1)
	 =  HappyAbsSyn35
		 (Function "leq" [happy_var_1, happy_var_3]
	)
happyReduction_90 _ _ _  = notHappyAtAll 

happyReduce_91 = happySpecReduce_3  35 happyReduction_91
happyReduction_91 (HappyAbsSyn35  happy_var_3)
	_
	(HappyAbsSyn35  happy_var_1)
	 =  HappyAbsSyn35
		 (Function "gt" [happy_var_1, happy_var_3]
	)
happyReduction_91 _ _ _  = notHappyAtAll 

happyReduce_92 = happySpecReduce_3  35 happyReduction_92
happyReduction_92 (HappyAbsSyn35  happy_var_3)
	_
	(HappyAbsSyn35  happy_var_1)
	 =  HappyAbsSyn35
		 (Function "and" [happy_var_1, happy_var_3]
	)
happyReduction_92 _ _ _  = notHappyAtAll 

happyReduce_93 = happySpecReduce_3  35 happyReduction_93
happyReduction_93 (HappyAbsSyn35  happy_var_3)
	_
	(HappyAbsSyn35  happy_var_1)
	 =  HappyAbsSyn35
		 (Function "or" [happy_var_1, happy_var_3]
	)
happyReduction_93 _ _ _  = notHappyAtAll 

happyReduce_94 = happySpecReduce_3  35 happyReduction_94
happyReduction_94 (HappyAbsSyn35  happy_var_3)
	_
	(HappyAbsSyn35  happy_var_1)
	 =  HappyAbsSyn35
		 (Function "geq" [happy_var_1, happy_var_3]
	)
happyReduction_94 _ _ _  = notHappyAtAll 

happyReduce_95 = happySpecReduce_3  35 happyReduction_95
happyReduction_95 (HappyAbsSyn35  happy_var_3)
	_
	(HappyAbsSyn35  happy_var_1)
	 =  HappyAbsSyn35
		 (Function "eq" [happy_var_1, happy_var_3]
	)
happyReduction_95 _ _ _  = notHappyAtAll 

happyReduce_96 = happySpecReduce_3  35 happyReduction_96
happyReduction_96 (HappyAbsSyn35  happy_var_3)
	_
	(HappyAbsSyn35  happy_var_1)
	 =  HappyAbsSyn35
		 (Function "not" [Function "eq" [happy_var_1, happy_var_3]]
	)
happyReduction_96 _ _ _  = notHappyAtAll 

happyReduce_97 = happySpecReduce_2  35 happyReduction_97
happyReduction_97 (HappyAbsSyn35  happy_var_2)
	_
	 =  HappyAbsSyn35
		 (Function "not" [happy_var_2]
	)
happyReduction_97 _ _  = notHappyAtAll 

happyReduce_98 = happySpecReduce_3  35 happyReduction_98
happyReduction_98 (HappyAbsSyn35  happy_var_3)
	_
	(HappyAbsSyn35  happy_var_1)
	 =  HappyAbsSyn35
		 (Function "plus" [happy_var_1, happy_var_3]
	)
happyReduction_98 _ _ _  = notHappyAtAll 

happyReduce_99 = happySpecReduce_3  35 happyReduction_99
happyReduction_99 (HappyAbsSyn35  happy_var_3)
	_
	(HappyAbsSyn35  happy_var_1)
	 =  HappyAbsSyn35
		 (Function "power" [happy_var_1, happy_var_3]
	)
happyReduction_99 _ _ _  = notHappyAtAll 

happyReduce_100 = happySpecReduce_3  35 happyReduction_100
happyReduction_100 (HappyAbsSyn35  happy_var_3)
	_
	(HappyAbsSyn35  happy_var_1)
	 =  HappyAbsSyn35
		 (Function "minus" [happy_var_1, happy_var_3]
	)
happyReduction_100 _ _ _  = notHappyAtAll 

happyReduce_101 = happySpecReduce_2  35 happyReduction_101
happyReduction_101 (HappyAbsSyn35  happy_var_2)
	_
	 =  HappyAbsSyn35
		 (Function "multiply" [Variable "-1", happy_var_2]
	)
happyReduction_101 _ _  = notHappyAtAll 

happyReduce_102 = happySpecReduce_3  35 happyReduction_102
happyReduction_102 (HappyAbsSyn35  happy_var_3)
	_
	(HappyAbsSyn35  happy_var_1)
	 =  HappyAbsSyn35
		 (Function "multiply" [happy_var_1, happy_var_3]
	)
happyReduction_102 _ _ _  = notHappyAtAll 

happyReduce_103 = happySpecReduce_3  35 happyReduction_103
happyReduction_103 (HappyAbsSyn35  happy_var_3)
	_
	(HappyAbsSyn35  happy_var_1)
	 =  HappyAbsSyn35
		 (Function "divide" [happy_var_1, happy_var_3]
	)
happyReduction_103 _ _ _  = notHappyAtAll 

happyReduce_104 = happyReduce 4 35 happyReduction_104
happyReduction_104 (_ `HappyStk`
	(HappyAbsSyn34  happy_var_3) `HappyStk`
	_ `HappyStk`
	(HappyTerminal (TokenString happy_var_1)) `HappyStk`
	happyRest)
	 = HappyAbsSyn35
		 (Function happy_var_1 (reverse happy_var_3)
	) `HappyStk` happyRest

happyReduce_105 = happySpecReduce_1  35 happyReduction_105
happyReduction_105 (HappyTerminal (TokenString happy_var_1))
	 =  HappyAbsSyn35
		 (Variable happy_var_1
	)
happyReduction_105 _  = notHappyAtAll 

happyReduce_106 = happySpecReduce_3  35 happyReduction_106
happyReduction_106 _
	(HappyAbsSyn35  happy_var_2)
	_
	 =  HappyAbsSyn35
		 (happy_var_2
	)
happyReduction_106 _ _ _  = notHappyAtAll 

happyReduce_107 = happySpecReduce_3  36 happyReduction_107
happyReduction_107 (HappyAbsSyn21  happy_var_3)
	_
	(HappyAbsSyn35  happy_var_1)
	 =  HappyAbsSyn36
		 ([(happy_var_1, happy_var_3)]
	)
happyReduction_107 _ _ _  = notHappyAtAll 

happyReduce_108 = happyReduce 5 36 happyReduction_108
happyReduction_108 ((HappyAbsSyn36  happy_var_5) `HappyStk`
	_ `HappyStk`
	(HappyAbsSyn21  happy_var_3) `HappyStk`
	_ `HappyStk`
	(HappyAbsSyn35  happy_var_1) `HappyStk`
	happyRest)
	 = HappyAbsSyn36
		 ((happy_var_1, happy_var_3) : happy_var_5
	) `HappyStk` happyRest

happyNewToken action sts stk
	= lexer(\tk -> 
	let cont i = action i i tk (HappyState action) sts stk in
	case tk of {
	TokenEOF -> action 82 82 tk (HappyState action) sts stk;
	TokenSum -> cont 37;
	TokenPSum -> cont 38;
	TokenComma -> cont 39;
	TokenSeq -> cont 40;
	TokenIndependence -> cont 41;
	TokenOSB -> cont 42;
	TokenCSB -> cont 43;
	TokenOB -> cont 44;
	TokenOr -> cont 45;
	TokenOA -> cont 46;
	TokenCA -> cont 47;
	TokenCB -> cont 48;
	TokenPlus -> cont 49;
	TokenPower -> cont 50;
	TokenPlusPlus -> cont 51;
	TokenAnd -> cont 52;
	TokenNot -> cont 53;
	TokenNotEqual -> cont 54;
	TokenMinus -> cont 55;
	TokenDivide -> cont 56;
	TokenMultiply -> cont 57;
	TokenColon -> cont 58;
	TokenBecomes -> cont 59;
	TokenSmaller -> cont 60;
	TokenGreater -> cont 61;
	TokenDotDot -> cont 62;
	TokenSmallerEq -> cont 63;
	TokenInit -> cont 64;
	TokenGreaterEq -> cont 65;
	TokenEqual -> cont 66;
	TokenImplies -> cont 67;
	TokenProbDef -> cont 68;
	TokenString happy_dollar_dollar -> cont 69;
	TokenHide -> cont 70;
	TokenRename -> cont 71;
	TokenBool -> cont 72;
	TokenActions -> cont 73;
	TokenEncap -> cont 74;
	TokenComm -> cont 75;
	TokenConstant -> cont 76;
	TokenFormula -> cont 77;
	TokenGlobal -> cont 78;
	TokenUntilFormula -> cont 79;
	TokenType -> cont 80;
	TokenFunction -> cont 81;
	_ -> happyError' tk
	})

happyError_ 82 tk = happyError' tk
happyError_ _ tk = happyError' tk

happyThen :: () => ParserMonad a -> (a -> ParserMonad b) -> ParserMonad b
happyThen = (thenParserMonad)
happyReturn :: () => a -> ParserMonad a
happyReturn = (returnParserMonad)
happyThen1 = happyThen
happyReturn1 :: () => a -> ParserMonad a
happyReturn1 = happyReturn
happyError' :: () => (Token) -> ParserMonad a
happyError' tk = parseError tk

parse = happySomeParser where
  happySomeParser = happyThen (happyParse action_0) (\x -> case x of {HappyAbsSyn4 z -> happyReturn z; _other -> notHappyAtAll })

happySeq = happyDontSeq


parseError :: Token -> ParserMonad a 
parseError t = failParserMonad ("Error: Parser did not expect token \"" ++ show t ++ "\"")


data Token = TokenSum
           | TokenSeq  
           | TokenString String
           | TokenIndependence
           | TokenUntilFormula
           | TokenOB 
           | TokenCB 
           | TokenOSB 
           | TokenCSB 
           | TokenOA
           | TokenCA
           | TokenOr
           | TokenImplies
           | TokenColon
           | TokenBool
           | TokenPlus
           | TokenPower
           | TokenPlusPlus
           | TokenMinus
           | TokenDivide
           | TokenMultiply
           | TokenThen
           | TokenElse
           | TokenActions
           | TokenComma
           | TokenEqual
           | TokenNotEqual
           | TokenNot
           | TokenPSum
           | TokenDotDot
           | TokenHide
           | TokenRename
           | TokenEncap
           | TokenAnd
           | TokenComm
           | TokenConstant
           | TokenFormula
           | TokenType
           | TokenGlobal
           | TokenFunction
           | TokenInit
           | TokenSmaller
           | TokenGreater
           | TokenSmallerEq
           | TokenGreaterEq
           | TokenProbDef
           | TokenBecomes
           | TokenEOF
instance Show Token where
  show x =
   case x of 
     TokenSum -> "sum("
     TokenSeq -> "."
     TokenString s -> s
     TokenIndependence -> "||"
     TokenUntilFormula -> "U"
     TokenOB  -> "("
     TokenCB  -> ")"
     TokenOSB -> "["
     TokenCSB -> "]"
     TokenOA  -> "{"
     TokenCA  -> "}"
     TokenOr -> "|"
     TokenImplies -> "=>"
     TokenColon -> ":"
     TokenBool -> "Bool"
     TokenPlus -> "+"
     TokenPower -> "^"
     TokenPlusPlus -> "++"
     TokenMinus -> "-"
     TokenDivide -> "/"
     TokenMultiply -> "*"
     TokenThen -> "<|"
     TokenElse -> "|>"
     TokenActions -> "actions"
     TokenComma -> ","
     TokenEqual -> "="
     TokenNotEqual -> "!="
     TokenNot -> "!"
     TokenPSum -> "psum"
     TokenDotDot -> ".."
     TokenHide -> "hide"
     TokenGlobal -> "global"
     TokenEncap -> "encap"
     TokenAnd -> "&"
     TokenComm -> "comm"
     TokenConstant -> "constant"
     TokenFormula -> "formula"
     TokenType -> "type"
     TokenFunction -> "function"
     TokenInit -> "init"
     TokenSmaller -> "<"
     TokenGreater -> ">"
     TokenSmallerEq -> "<="
     TokenGreaterEq -> ">="
     TokenProbDef -> "->"
     TokenBecomes -> ":="
     TokenEOF -> "EOF"

-- lexer :: String -> [Token] 
lexer :: (Token -> ParserMonad a) -> ParserMonad a
lexer cont ('\n':cs) = \line -> lexer cont cs (line + 1)
lexer cont ('\r':cs) = \line -> lexer cont cs line
lexer cont [] = cont TokenEOF [] 
lexer cont ('-':'-':cs) = lexComment cont (cs)
lexer cont ('.':'.':cs) = cont TokenDotDot cs
lexer cont ('i':'n':'i':'t':' ':cs) = cont TokenInit cs
lexer cont ('i':'n':'i':'t':'\n':cs) = cont TokenInit ('\n':cs)
lexer cont ('i':'n':'i':'t':'\r':cs) = cont TokenInit ('\r':cs)
lexer cont ('i':'n':'i':'t':'\t':cs) = cont TokenInit ('\t':cs)
lexer cont ('u':'n':'t':'i':'l':' ':cs) = cont TokenUntilFormula cs
lexer cont ('u':'n':'t':'i':'l':'\n':cs) = cont TokenUntilFormula ('\n':cs)
lexer cont ('u':'n':'t':'i':'l':'\r':cs) = cont TokenUntilFormula ('\r':cs)
lexer cont ('u':'n':'t':'i':'l':'\t':cs) = cont TokenUntilFormula ('\t':cs)
lexer cont ('b':'o':'o':'l':' ':cs) = cont TokenBool cs
lexer cont ('b':'o':'o':'l':'\n':cs) = cont TokenBool ('\n':cs)
lexer cont ('b':'o':'o':'l':'\r':cs) = cont TokenBool ('\r':cs)
lexer cont ('b':'o':'o':'l':'\t':cs) = cont TokenBool ('\t':cs)
lexer cont ('o':'b':'s':'e':'r':'v':'e':' ':cs) = cont TokenActions cs
lexer cont ('o':'b':'s':'e':'r':'v':'e':'\n':cs) = cont TokenActions ('\n':cs)
lexer cont ('o':'b':'s':'e':'r':'v':'e':'\r':cs) = cont TokenActions ('\r':cs)
lexer cont ('o':'b':'s':'e':'r':'v':'e':'\t':cs) = cont TokenActions ('\t':cs)
lexer cont ('h':'i':'d':'e':' ':cs) = cont TokenHide cs
lexer cont ('h':'i':'d':'e':'\n':cs) = cont TokenHide ('\n':cs)
lexer cont ('h':'i':'d':'e':'\r':cs) = cont TokenHide ('\r':cs)
lexer cont ('h':'i':'d':'e':'\t':cs) = cont TokenHide ('\t':cs)
lexer cont ('h':'i':'d':'e':'(':cs) = cont TokenHide ('(':cs)

lexer cont ('g':'l':'o':'b':'a':'l':' ':cs) = cont TokenGlobal cs
lexer cont ('g':'l':'o':'b':'a':'l':'\n':cs) = cont TokenGlobal ('\n':cs)
lexer cont ('g':'l':'o':'b':'a':'l':'\r':cs) = cont TokenGlobal ('\r':cs)
lexer cont ('g':'l':'o':'b':'a':'l':'\t':cs) = cont TokenGlobal ('\t':cs)
lexer cont ('g':'l':'o':'b':'a':'l':'(':cs) = cont TokenGlobal ('(':cs)

lexer cont ('s':'u':'m':cs) | not (isAlpha (head cs)) = cont TokenSum cs
lexer cont ('p':'s':'u':'m':cs) | not (isAlpha (head cs)) = cont TokenPSum cs
lexer cont ('c':'o':'n':'s':'t':'a':'n':'t':' ':cs) = cont TokenConstant cs
lexer cont ('c':'o':'n':'s':'t':'a':'n':'t':'\n':cs) = cont TokenConstant ('\n':cs)
lexer cont ('c':'o':'n':'s':'t':'a':'n':'t':'\r':cs) = cont TokenConstant ('\r':cs)
lexer cont ('c':'o':'n':'s':'t':'a':'n':'t':'\t':cs) = cont TokenConstant ('\t':cs)
lexer cont ('f':'o':'r':'m':'u':'l':'a':' ':cs) = cont TokenFormula cs
lexer cont ('f':'o':'r':'m':'u':'l':'a':'\n':cs) = cont TokenFormula ('\n':cs)
lexer cont ('f':'o':'r':'m':'u':'l':'a':'\r':cs) = cont TokenFormula ('\r':cs)
lexer cont ('f':'o':'r':'m':'u':'l':'a':'\t':cs) = cont TokenFormula ('\t':cs)
lexer cont ('t':'y':'p':'e':' ':cs) = cont TokenType cs
lexer cont ('t':'y':'p':'e':'\n':cs) = cont TokenType ('\n':cs)
lexer cont ('t':'y':'p':'e':'\r':cs) = cont TokenType ('\r':cs)
lexer cont ('t':'y':'p':'e':'\t':cs) = cont TokenType ('\t':cs)
lexer cont ('f':'u':'n':'c':'t':'i':'o':'n':' ':cs) = cont TokenFunction cs
lexer cont ('f':'u':'n':'c':'t':'i':'o':'n':'\n':cs) = cont TokenFunction ('\n':cs)
lexer cont ('f':'u':'n':'c':'t':'i':'o':'n':'\r':cs) = cont TokenFunction ('\r':cs)
lexer cont ('f':'u':'n':'c':'t':'i':'o':'n':'\t':cs) = cont TokenFunction ('\t':cs)
lexer cont ('e':'n':'c':'a':'p':' ':cs) = cont TokenEncap cs
lexer cont ('e':'n':'c':'a':'p':'\n':cs) = cont TokenEncap ('\n':cs)
lexer cont ('e':'n':'c':'a':'p':'\r':cs) = cont TokenEncap ('\r':cs)
lexer cont ('e':'n':'c':'a':'p':'\t':cs) = cont TokenEncap ('\t':cs)
lexer cont ('e':'n':'c':'a':'p':'(':cs) = cont TokenEncap ('(':cs)
lexer cont ('r':'e':'n':'a':'m':'e':' ':cs) = cont TokenRename cs
lexer cont ('r':'e':'n':'a':'m':'e':'\n':cs) = cont TokenRename ('\n':cs)
lexer cont ('r':'e':'n':'a':'m':'e':'\r':cs) = cont TokenRename ('\r':cs)
lexer cont ('r':'e':'n':'a':'m':'e':'\t':cs) = cont TokenRename ('\t':cs)
lexer cont ('r':'e':'n':'a':'m':'e':'(':cs) = cont TokenRename ('(':cs)
lexer cont ('c':'o':'m':'m':' ':cs) = cont TokenComm cs
lexer cont ('c':'o':'m':'m':'\n':cs) = cont TokenComm ('\n':cs)
lexer cont ('c':'o':'m':'m':'\r':cs) = cont TokenComm ('\r':cs)
lexer cont ('c':'o':'m':'m':'\t':cs) = cont TokenComm ('\t':cs)
lexer cont (c:cs) 
      | isSpace c = lexer cont cs 
      | (\x -> isDigit x) c = lexNumber cont (c:cs)
      | (\x -> isAlpha x) c = lexString cont (c:cs)
lexer cont ('.':cs) = cont TokenSeq cs 
lexer cont ('|':'|':cs) = cont TokenIndependence cs
lexer cont ('|':'>':cs) = cont TokenElse cs 
lexer cont ('<':'|':cs) = cont TokenThen cs 
lexer cont ('&':cs) = cont TokenAnd cs 
lexer cont ('<':'=':cs) = cont TokenSmallerEq cs 
lexer cont ('>':'=':cs) = cont TokenGreaterEq cs 
lexer cont ('<':cs) = cont TokenSmaller cs 
lexer cont ('>':cs) = cont TokenGreater cs 
lexer cont ('(':cs) = cont TokenOB cs 
lexer cont ('|':cs) = cont TokenOr cs 
lexer cont (')':cs) = cont TokenCB cs 
lexer cont ('[':cs) = cont TokenOSB cs 
lexer cont (']':cs) = cont TokenCSB cs 
lexer cont ('{':cs) = cont TokenOA cs 
lexer cont ('}':cs) = cont TokenCA cs 
lexer cont ('+':'+':cs) = cont TokenPlusPlus cs
lexer cont ('-':'>':cs) = cont TokenProbDef cs
lexer cont (',':cs) = cont TokenComma cs
lexer cont ('+':cs) = cont TokenPlus cs
lexer cont ('^':cs) = cont TokenPower cs
lexer cont ('-':cs) = cont TokenMinus cs
lexer cont ('!':'=':cs) = cont TokenNotEqual cs
lexer cont ('!':cs) = cont TokenNot cs
lexer cont ('/':cs) = cont TokenDivide cs
lexer cont ('*':cs) = cont TokenMultiply cs
lexer cont (':':'=':cs) = cont TokenBecomes cs
lexer cont (':':cs) = cont TokenColon cs
lexer cont ('=':'>':cs) = cont TokenImplies cs
lexer cont ('=':cs) = cont TokenEqual cs
lexer cont s = \line -> error("Error: Lexer could not deal with character \"" ++ [head s] ++ "\" on line " ++ show line)

lexComment cont cs = lexer cont rest
  where
    rest = dropWhile (\x -> x /= '\n' && x /= '\r') cs
lexString cont cs = cont (TokenString var) rest
  where
    (var,rest) = getString cs
lexNumber cont cs = cont (TokenString var) rest
  where
    (var,rest) = getNumber cs

getString [] = ("","")
getString ('.':'.':cs) = ("", '.':'.':cs)
getString (c:cs) | elem c "\n\t\r ',()&=*.<>+{}^:-!?;/|\"[]" = ("", c:cs)
                 | otherwise = (str,rest)
  where
    (s,r) = getString cs
    str   = c:s
    rest  = r

getNumber [] = ("","")
getNumber ('.':'.':cs) = ("", '.':'.':cs)
getNumber (c:cs) | elem c "\n\t\r ',()&=*<>+_{}:^-!?;/|\"[]" = ("", c:cs)
                 | otherwise = (str,rest)
  where
    (s,r) = getNumber cs
    str   = c:s
    rest  = r
{-# LINE 1 "templates/GenericTemplate.hs" #-}
{-# LINE 1 "templates/GenericTemplate.hs" #-}
{-# LINE 1 "<built-in>" #-}
{-# LINE 1 "<command-line>" #-}
{-# LINE 1 "templates/GenericTemplate.hs" #-}
-- Id: GenericTemplate.hs,v 1.26 2005/01/14 14:47:22 simonmar Exp 

{-# LINE 30 "templates/GenericTemplate.hs" #-}








{-# LINE 51 "templates/GenericTemplate.hs" #-}

{-# LINE 61 "templates/GenericTemplate.hs" #-}

{-# LINE 70 "templates/GenericTemplate.hs" #-}

infixr 9 `HappyStk`
data HappyStk a = HappyStk a (HappyStk a)

-----------------------------------------------------------------------------
-- starting the parse

happyParse start_state = happyNewToken start_state notHappyAtAll notHappyAtAll

-----------------------------------------------------------------------------
-- Accepting the parse

-- If the current token is (1), it means we've just accepted a partial
-- parse (a %partial parser).  We must ignore the saved token on the top of
-- the stack in this case.
happyAccept (1) tk st sts (_ `HappyStk` ans `HappyStk` _) =
	happyReturn1 ans
happyAccept j tk st sts (HappyStk ans _) = 
	 (happyReturn1 ans)

-----------------------------------------------------------------------------
-- Arrays only: do the next action

{-# LINE 148 "templates/GenericTemplate.hs" #-}

-----------------------------------------------------------------------------
-- HappyState data type (not arrays)



newtype HappyState b c = HappyState
        (Int ->                    -- token number
         Int ->                    -- token number (yes, again)
         b ->                           -- token semantic value
         HappyState b c ->              -- current state
         [HappyState b c] ->            -- state stack
         c)



-----------------------------------------------------------------------------
-- Shifting a token

happyShift new_state (1) tk st sts stk@(x `HappyStk` _) =
     let (i) = (case x of { HappyErrorToken (i) -> i }) in
--     trace "shifting the error token" $
     new_state i i tk (HappyState (new_state)) ((st):(sts)) (stk)

happyShift new_state i tk st sts stk =
     happyNewToken new_state ((st):(sts)) ((HappyTerminal (tk))`HappyStk`stk)

-- happyReduce is specialised for the common cases.

happySpecReduce_0 i fn (1) tk st sts stk
     = happyFail (1) tk st sts stk
happySpecReduce_0 nt fn j tk st@((HappyState (action))) sts stk
     = action nt j tk st ((st):(sts)) (fn `HappyStk` stk)

happySpecReduce_1 i fn (1) tk st sts stk
     = happyFail (1) tk st sts stk
happySpecReduce_1 nt fn j tk _ sts@(((st@(HappyState (action))):(_))) (v1`HappyStk`stk')
     = let r = fn v1 in
       happySeq r (action nt j tk st sts (r `HappyStk` stk'))

happySpecReduce_2 i fn (1) tk st sts stk
     = happyFail (1) tk st sts stk
happySpecReduce_2 nt fn j tk _ ((_):(sts@(((st@(HappyState (action))):(_))))) (v1`HappyStk`v2`HappyStk`stk')
     = let r = fn v1 v2 in
       happySeq r (action nt j tk st sts (r `HappyStk` stk'))

happySpecReduce_3 i fn (1) tk st sts stk
     = happyFail (1) tk st sts stk
happySpecReduce_3 nt fn j tk _ ((_):(((_):(sts@(((st@(HappyState (action))):(_))))))) (v1`HappyStk`v2`HappyStk`v3`HappyStk`stk')
     = let r = fn v1 v2 v3 in
       happySeq r (action nt j tk st sts (r `HappyStk` stk'))

happyReduce k i fn (1) tk st sts stk
     = happyFail (1) tk st sts stk
happyReduce k nt fn j tk st sts stk
     = case happyDrop (k - ((1) :: Int)) sts of
	 sts1@(((st1@(HappyState (action))):(_))) ->
        	let r = fn stk in  -- it doesn't hurt to always seq here...
       		happyDoSeq r (action nt j tk st1 sts1 r)

happyMonadReduce k nt fn (1) tk st sts stk
     = happyFail (1) tk st sts stk
happyMonadReduce k nt fn j tk st sts stk =
        happyThen1 (fn stk tk) (\r -> action nt j tk st1 sts1 (r `HappyStk` drop_stk))
       where (sts1@(((st1@(HappyState (action))):(_)))) = happyDrop k ((st):(sts))
             drop_stk = happyDropStk k stk

happyMonad2Reduce k nt fn (1) tk st sts stk
     = happyFail (1) tk st sts stk
happyMonad2Reduce k nt fn j tk st sts stk =
       happyThen1 (fn stk tk) (\r -> happyNewToken new_state sts1 (r `HappyStk` drop_stk))
       where (sts1@(((st1@(HappyState (action))):(_)))) = happyDrop k ((st):(sts))
             drop_stk = happyDropStk k stk





             new_state = action


happyDrop (0) l = l
happyDrop n ((_):(t)) = happyDrop (n - ((1) :: Int)) t

happyDropStk (0) l = l
happyDropStk n (x `HappyStk` xs) = happyDropStk (n - ((1)::Int)) xs

-----------------------------------------------------------------------------
-- Moving to a new state after a reduction

{-# LINE 246 "templates/GenericTemplate.hs" #-}
happyGoto action j tk st = action j j tk (HappyState action)


-----------------------------------------------------------------------------
-- Error recovery ((1) is the error token)

-- parse error if we are in recovery and we fail again
happyFail (1) tk old_st _ stk@(x `HappyStk` _) =
     let (i) = (case x of { HappyErrorToken (i) -> i }) in
--	trace "failing" $ 
        happyError_ i tk

{-  We don't need state discarding for our restricted implementation of
    "error".  In fact, it can cause some bogus parses, so I've disabled it
    for now --SDM

-- discard a state
happyFail  (1) tk old_st (((HappyState (action))):(sts)) 
						(saved_tok `HappyStk` _ `HappyStk` stk) =
--	trace ("discarding state, depth " ++ show (length stk))  $
	action (1) (1) tk (HappyState (action)) sts ((saved_tok`HappyStk`stk))
-}

-- Enter error recovery: generate an error token,
--                       save the old token and carry on.
happyFail  i tk (HappyState (action)) sts stk =
--      trace "entering error recovery" $
	action (1) (1) tk (HappyState (action)) sts ( (HappyErrorToken (i)) `HappyStk` stk)

-- Internal happy errors:

notHappyAtAll :: a
notHappyAtAll = error "Internal Happy error\n"

-----------------------------------------------------------------------------
-- Hack to get the typechecker to accept our action functions







-----------------------------------------------------------------------------
-- Seq-ing.  If the --strict flag is given, then Happy emits 
--	happySeq = happyDoSeq
-- otherwise it emits
-- 	happySeq = happyDontSeq

happyDoSeq, happyDontSeq :: a -> b -> b
happyDoSeq   a b = a `seq` b
happyDontSeq a b = b

-----------------------------------------------------------------------------
-- Don't inline any functions from the template.  GHC has a nasty habit
-- of deciding to inline happyGoto everywhere, which increases the size of
-- the generated parser quite a bit.

{-# LINE 312 "templates/GenericTemplate.hs" #-}
{-# NOINLINE happyShift #-}
{-# NOINLINE happySpecReduce_0 #-}
{-# NOINLINE happySpecReduce_1 #-}
{-# NOINLINE happySpecReduce_2 #-}
{-# NOINLINE happySpecReduce_3 #-}
{-# NOINLINE happyReduce #-}
{-# NOINLINE happyMonadReduce #-}
{-# NOINLINE happyGoto #-}
{-# NOINLINE happyFail #-}

-- end of Happy Template.
