module ToAUT where

import ToPA
import LPPE
import Confluence
import DataSpec
import Ratio
import List

-------------------------------
-- Translation to AUT format --
-------------------------------

data OutputString = OutputString [Char]
instance Show OutputString where
   show (OutputString s) = s


toAUTNonProb :: Bool -> PSpecification -> Bool -> ConfluentSummands -> Bool -> Bool -> Bool -> OutputString
toAUTNonProb ignorecycles spec cadp confl checkDTMC isMA removeRates = OutputString ("des (0," ++ show (length transitions2) ++ "," ++ show (length states) ++ ")\n" ++ (printTransitionsNonProb cadp transitions2))
  where
    (statespace,initial,_) = getStateSpace ignorecycles spec confl False checkDTMC isMA removeRates
    states                 = fst statespace
    transitions            = snd statespace
    transitions2           = createTransitionsNonProb transitions

createTransitionsNonProb :: [(Int, EdgeLabel, [(Probability, Int)])] -> [(Int, EdgeLabel, Int)]
createTransitionsNonProb []                              = []
createTransitionsNonProb ((from, label, []):trans)       = createTransitionsNonProb trans
createTransitionsNonProb ((from, label, (to:tos)):trans) = (from, label, snd to):rest
  where
    rest = createTransitionsNonProb ((from, label, tos):trans)

printTransitionsNonProb cadp [] = []
printTransitionsNonProb cadp ((from, label, to):ts) = "(" ++ show from ++ ",\"" ++ (if cadp then (if label == "tau" then "i" else changeCommas label) else label) ++ "\"," ++ show to ++ ")\n" ++ (printTransitionsNonProb cadp ts)

toAUTProb :: Bool -> PSpecification -> Bool -> ConfluentSummands -> Bool -> Bool -> Bool -> OutputString
toAUTProb ignorecycles spec cadp confl checkDTMC isMA removeRates = OutputString ("des (0, " ++ show (length transitions + length transitions2 - probOneTrans) ++ ", " ++ show (length states + length transitions - probOneTrans) ++ ")\n" ++ (printTransitionsProb cadp transitions (length states)))
    where
     (statespace,initial,_) = getStateSpace ignorecycles spec confl False checkDTMC isMA removeRates
     states                 = fst statespace
     transitions            = snd statespace
     transitions2           = createTransitionsNonProb transitions
     probOneTrans           = length ["" | (f,l,t) <- transitions, length t == 1]

printTransitionsProb :: Bool -> [(Int, EdgeLabel, [(Probability, Int)])] -> Int -> [Char]
printTransitionsProb cadp [] i                                  = []
printTransitionsProb cadp ((from, label, [("1",to)]):ts) i      = "(" ++ show from ++ ", " ++ (if cadp then (if label == "tau" then "i" else changeCommas label) else label) ++ ", " ++ show to ++ ")\n"
                                                                  ++ printTransitionsProb cadp ts i
printTransitionsProb cadp ((from, label, ((prob,to):tos)):ts) i = printTransitionsProb2 cadp (from, label, ((prob,to):tos)) i
                                                                  ++ printTransitionsProb cadp ts (i+1)

printTransitionsProb2 :: Bool -> (Int, EdgeLabel, [(Probability, Int)]) -> Int -> [Char]
printTransitionsProb2 cadp (from, label, []) i              = "(" ++ show from ++ ", " ++ (if cadp then (if label == "tau" then "i" else changeCommas label) else label) ++ ", " ++ show i ++ ")\n"
printTransitionsProb2 cadp (from, label, ((prob,to):tos)) i = "(" ++ show i ++ ", \"prob " ++ (if cadp then printFraction prob else prob) ++ "\", " ++ show to ++ ")\n" ++ (printTransitionsProb2 cadp (from, label, tos) i)
  where
    changeFraction = \x -> (toInteger (numerator x)) % (toInteger (denominator x))
    printFraction  = \x -> show (fromRational (changeFraction (getFraction prob)))

changeCommas ""       = ""
changeCommas (',':y) = ';':(changeCommas y)
changeCommas (x:y)    = x:(changeCommas y)


--printTransitionsProb cadp ((from, label, []):ts) i = "(" ++ show from ++ ",\"" ++ (if cadp then (if label == "tau" then "i" else label) else label) ++ "\"," ++ show i ++ ")\n" ++ (printTransitionsProb cadp ts (i+1))



--printTransitionsProb :: Bool -> [(Int, EdgeLabel, [(Probability, Int)])] -> Int -> [Char]
--printTransitionsProb cadp [] i                                  = []
--printTransitionsProb cadp ((from, label, []):ts) i              = "(" ++ show from ++ ", " ++ (if cadp then (if label == "tau" then "i" else label) else label) ++ ", " ++ show i ++ ")\n" ++ (printTransitionsProb cadp ts (i+1))
--printTransitionsProb cadp ((from, label, ((prob,to):tos)):ts) i = "(" ++ show i ++ ", \"prob " ++ (if cadp then printFraction prob else prob) ++ "\", " ++ show to ++ ")\n" ++ (printTransitionsProb cadp ((from, label, tos):ts) i)
--  where
--    changeFraction = \x -> (toInteger (numerator x)) % (toInteger (denominator x))
--    printFraction  = \x -> show (fromRational (changeFraction (getFraction prob)))
--printTransitionsProb cadp ((from, label, []):ts) i = "(" ++ show from ++ ",\"" ++ (if cadp then (if label == "tau" then "i" else label) else label) ++ "\"," ++ show i ++ ")\n" ++ (printTransitionsProb cadp ts (i+1))

toAUTDTMC :: Bool -> PSpecification -> Bool -> ConfluentSummands -> Bool -> Bool -> Bool -> OutputString
toAUTDTMC ignorecycles spec cadp confl checkDTMC isMA removeRates = OutputString ("des (0, " ++ show (length transitions2) ++ ", " ++ show (length states) ++ ")\n" ++ (printTransitionsDTMC cadp transitions))
    where
     (statespace,initial,_) = getStateSpace ignorecycles spec confl False checkDTMC isMA removeRates
     states                 = fst statespace
     transitions            = snd statespace
     transitions2           = createTransitionsNonProb transitions

printTransitionsDTMC :: Bool -> [(Int, EdgeLabel, [(Probability, Int)])] -> [Char] 
printTransitionsDTMC cadp []                                  = []
printTransitionsDTMC cadp ((from, label, []):ts)              = printTransitionsDTMC cadp ts
printTransitionsDTMC cadp ((from, label, ((prob,to):tos)):ts) = printTransitionsDTMC2 cadp ((from, label, outgoing):ts)
    where
	 outgoing = sortBy order ((prob,to):tos)

order :: (Probability, Int) -> (Probability, Int) -> Ordering
order (p1,t1) (p2,t2) | t1 < t2   = LT
                      | otherwise = GT
	
printTransitionsDTMC2 :: Bool -> [(Int, EdgeLabel, [(Probability, Int)])] -> [Char] 
printTransitionsDTMC2 cadp ((from, label, []):ts)              = printTransitionsDTMC cadp ts
printTransitionsDTMC2 cadp ((from, label, ((prob,to):tos)):ts) = "(" ++ show from ++ ", \"" ++ (if cadp then (if label == "tau" then "i" else changeCommas label) else label) ++ "; prob " ++ (if cadp then printFraction prob else prob) ++ "\", " ++ show to ++ ")\n" ++ (printTransitionsDTMC2 cadp ((from, label, tos):ts))
  where
    changeFraction = \x -> (toInteger (numerator x)) % (toInteger (denominator x))
    printFraction  = \x -> show (fromRational (changeFraction (getFraction prob)))

--toAUTMA :: Bool -> PSpecification -> Bool -> ConfluentSummands -> Bool -> OutputString
--toAUTMA ignorecycles spec cadp confl = OutputString ("des (0, " ++ show (length transitions + length transitions2) ++ ", " ++ show (length states + length transitions) ++ ")\n" ++ (printTransitionsProb cadp transitions (length states)))
--    where
--     (statespace,initial,_) = getStateSpace ignorecycles spec confl False False
--     states                 = fst statespace
--     transitions            = snd statespace
--     transitions2           = createTransitionsNonProb transitions

